@@ -4,21 +4,20 @@ namespace Icinga\Module\Director\Controllers;
 
 use Exception;
 use Icinga\Module\Director\Db\Migrations;
+use Icinga\Module\Director\Forms\ApplyMigrationsForm;
 
 class IndexController extends DashboardController
 {
+    protected $hasDeploymentEndpoint;
+
     public function indexAction()
     {
-        $this->view->dashboards = array();
-
-        $this->setViewScript('dashboard/index');
-
         if ($this->Config()->get('db', 'resource')) {
             $migrations = new Migrations($this->db());
 
             if ($migrations->hasSchema()) {
                 if (!$this->hasDeploymentEndpoint()) {
-                    $this->showKickstartForm();
+                    $this->showKickstartForm(false);
                 }
             } else {
                 $this->showKickstartForm();
@@ -26,10 +25,11 @@ class IndexController extends DashboardController
             }
 
             if ($migrations->hasPendingMigrations()) {
-                $this->view->form = $this
-                    ->loadForm('applyMigrations')
-                    ->setMigrations($migrations)
-                    ->handleRequest();
+                $this->content()->prepend(
+                    ApplyMigrationsForm::load()
+                        ->setMigrations($migrations)
+                        ->handleRequest()
+                );
             }
 
             parent::indexAction();
@@ -38,20 +38,25 @@ class IndexController extends DashboardController
         }
     }
 
-    protected function showKickstartForm()
+    protected function showKickstartForm($showTab = true)
     {
-        $this->singleTab($this->translate('Kickstart'));
-        $this->view->form = $this->loadForm('kickstart')->handleRequest();
+        if ($showTab) {
+            $this->addSingleTab($this->translate('Kickstart'));
+        }
+
+        $this->content()->prepend(
+            $this->loadForm('kickstart')->handleRequest()
+        );
     }
 
     protected function hasDeploymentEndpoint()
     {
         try {
-            $this->view->hasDeploymentEndpoint = $this->db()->hasDeploymentEndpoint();
+            $this->hasDeploymentEndpoint = $this->db()->hasDeploymentEndpoint();
         } catch (Exception $e) {
             return false;
         }
 
-        return $this->view->hasDeploymentEndpoint;
+        return $this->hasDeploymentEndpoint;
     }
 }
