@@ -16,12 +16,20 @@ class IcingaNotificationForm extends DirectorObjectForm
             return;
         }
 
-        $this->addElement('text', 'object_name', array(
-            'label'       => $this->translate('Notification'),
-            'required'    => true,
-            'description' => $this->translate('Icinga object name for this notification')
-        ));
- 
+        if ($this->isTemplate()) {
+            $this->addElement('text', 'object_name', array(
+                'label'       => $this->translate('Notification Template'),
+                'required'    => true,
+                'description' => $this->translate('Name for the Icinga notification template you are going to create')
+            ));
+        } else {
+            $this->addElement('text', 'object_name', array(
+                'label'       => $this->translate('Notification'),
+                'required'    => true,
+                'description' => $this->translate('Name for the Icinga notification you are going to create')
+            ));
+        }
+
         $this->addDisabledElement()
              ->addImportsElement()
              ->addUsersElement()
@@ -33,10 +41,31 @@ class IcingaNotificationForm extends DirectorObjectForm
              ->addDisabledElement()
              ->addCommandElements()
              ->addEventFilterElements()
+             ->addZoneElements()
              ->groupMainProperties()
              ->setButtons();
     }
 
+    protected function addZoneElements()
+    {
+        if (! $this->isTemplate()) {
+            return $this;
+        }
+
+        $this->addZoneElement();
+        $this->addDisplayGroup(array('zone_id'), 'clustering', array(
+            'decorators' => array(
+                'FormElements',
+                array('HtmlTag', array('tag' => 'dl')),
+                'Fieldset',
+            ),
+            'order' => 80,
+            'legend' => $this->translate('Zone settings')
+        ));
+
+        return $this;
+    }
+
     /**
      * @return self
      */
@@ -63,23 +92,20 @@ class IcingaNotificationForm extends DirectorObjectForm
 
         $applyTo = $this->getSentOrObjectValue('apply_to');
 
-        if ($applyTo === 'host') {
-            $columns = IcingaHost::enumProperties($this->db, 'host.');
-        } elseif ($applyTo === 'service') {
-            // TODO: Also add host properties
-            $columns = IcingaService::enumProperties($this->db, 'service.');
-        } else {
+        if (! $applyTo) {
             return $this;
         }
 
-        $this->addAssignFilter(array(
-            'columns' => $columns,
+        $suggestionContext = ucfirst($applyTo) . 'FilterColumns';
+        $this->addAssignFilter([
             'required' => true,
+            'suggestionContext' => $suggestionContext,
             'description' => $this->translate(
                 'This allows you to configure an assignment filter. Please feel'
                 . ' free to combine as many nested operators as you want'
             )
-        ));
+        ]);
+
         return $this;
     }
 
