@@ -3,18 +3,23 @@
 namespace Icinga\Module\Director\Forms;
 
 use Icinga\Data\Filter\Filter;
+use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Director\Data\PropertiesFilter\ArrayCustomVariablesFilter;
 use Icinga\Module\Director\Exception\NestingError;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaService;
 use Icinga\Module\Director\Objects\IcingaServiceSet;
+use Icinga\Module\Director\Web\Form\Validate\NamePattern;
+use dipl\Html\Html;
+use dipl\Html\Link;
 
 class IcingaServiceForm extends DirectorObjectForm
 {
     /** @var IcingaHost */
     private $host;
 
+    /** @var IcingaServiceSet */
     private $set;
 
     private $apply;
@@ -22,6 +27,7 @@ class IcingaServiceForm extends DirectorObjectForm
     /** @var IcingaService */
     protected $object;
 
+    /** @var IcingaService */
     private $applyGenerated;
 
     private $inheritedFrom;
@@ -68,7 +74,7 @@ class IcingaServiceForm extends DirectorObjectForm
 
     protected function providesOverrides()
     {
-        return  $this->applyGenerated
+        return $this->applyGenerated
             || $this->inheritedFrom
             || ($this->host && $this->set)
             || ($this->object && $this->object->usesVarOverrides());
@@ -80,16 +86,13 @@ class IcingaServiceForm extends DirectorObjectForm
             return;
         }
 
-        $this->addHtmlHint(
-            $this->getOverrideHint(),
-            array('name' => 'inheritance_hint')
-        );
+        $this->addOverrideHint();
 
         $group = $this->getDisplayGroup('custom_fields');
 
         if ($group) {
             $elements = $group->getElements();
-            $group->setElements(array($this->getElement('inheritance_hint')));
+            $group->setElements([$this->getElement('inheritance_hint')]);
             $group->addElements($elements);
             $this->setSubmitLabel(
                 $this->translate('Override vars')
@@ -120,6 +123,8 @@ class IcingaServiceForm extends DirectorObjectForm
     {
         if ($this->object) {
             $this->addHidden('object_type', $this->object->object_type);
+        } elseif ($this->preferredObjectType) {
+            $this->addHidden('object_type', $this->preferredObjectType);
         } else {
             $this->addHidden('object_type', 'template');
         }
@@ -127,6 +132,7 @@ class IcingaServiceForm extends DirectorObjectForm
         $this->addNameElement()
              ->addHostObjectElement()
              ->addImportsElement()
+             ->addChoices('service')
              ->addGroupsElement()
              ->addDisabledElement()
              ->addApplyForElement()
@@ -139,91 +145,94 @@ class IcingaServiceForm extends DirectorObjectForm
              ->setButtons();
     }
 
-    protected function getOverrideHint()
+    protected function addOverrideHint()
     {
-        $view = $this->getView();
-
         if ($this->object && $this->object->usesVarOverrides()) {
-            return $this->translate(
+            $hint = $this->translate(
                 'This service has been generated in an automated way, but still'
                 . ' allows you to override the following properties in a safe way.'
             );
-        }
-
-        if ($this->applyGenerated) {
-            return $view->escape(sprintf(
+        } elseif ($apply = $this->applyGenerated) {
+            $hint = Html::sprintf(
                 $this->translate(
-                    'This service has been generated using an apply rule, assigned where %s'
+                    'This service has been generated using the %s apply rule, assigned where %s'
                 ),
-                Filter::fromQueryString($this->applyGenerated->assign_filter)
-            ));
-        }
-
-        if ($this->host && $this->set) {
-            return $this->translate(
-                'This service belongs to a Service Set. Still, you might want'
-                . ' to override the following properties for this host only.'
+                Link::create(
+                    $apply->getObjectName(),
+                    'director/service',
+                    ['id' => $apply->get('id')],
+                    ['data-base-target' => '_next']
+                ),
+                (string) Filter::fromQueryString($apply->assign_filter)
             );
-        }
-
-        if ($this->inheritedFrom) {
-            $msg = $view->escape($this->translate(
+        } elseif ($this->host && $this->set) {
+            $hint = Html::sprintf(
+                $this->translate(
+                    'This service belongs to the %s Service Set. Still, you might want'
+                    . ' to override the following properties for this host only.'
+                ),
+                Link::create(
+                    $this->set->getObjectName(),
+                    'director/serviceset',
+                    ['id' => $this->set->get('id')],
+                    ['data-base-target' => '_next']
+                )
+            );
+        } elseif ($this->inheritedFrom) {
+            $msg = $this->translate(
                 'This service has been inherited from %s. Still, you might want'
                 . ' to change the following properties for this host only.'
-            ));
-
-            $name = $this->inheritedFrom;
-            $link = $view->qlink(
-                $name,
-                'director/service',
-                array(
-                    'host' => $name,
-                    'name' => $this->object->object_name,
-                ),
-                array('data-base-target' => '_next')
             );
 
-            return sprintf($msg, $link);
+            $name = $this->inheritedFrom;
+            $link = Link::create(
+                $name,
+                'director/service',
+                [
+                    'host' => $name,
+                    'name' => $this->object->getObjectName(),
+                ],
+                ['data-base-target' => '_next']
+            );
+
+            $hint = Html::sprintf($msg, $link);
+        } else {
+            throw new ProgrammingError('Got no override hint for your situation');
         }
 
-        $this->setSubmitLabel(
-            $this->translate('Override vars')
-        );
+        $this->setSubmitLabel($this->translate('Override vars'));
+
+        $this->addHtmlHint($hint, ['name' => 'inheritance_hint']);
     }
 
     protected function setupOnHostForSet()
     {
-        $view = $this->getView();
-        $msg = $view->escape($this->translate(
+        $msg = $this->translate(
             'This service belongs to the service set "%s". Still, you might want'
             . ' to change the following properties for this host only.'
-        ));
+        );
 
         $name = $this->set->getObjectName();
-        $link = $view->qlink(
+        $link = Link::create(
             $name,
             'director/serviceset',
-            array(
-                'name' => $name,
-            ),
-            array('data-base-target' => '_next')
+            ['name' => $name],
+            ['data-base-target' => '_next']
         );
 
         $this->addHtmlHint(
-            sprintf($msg, $link),
-            array('name' => 'inheritance_hint')
+            Html::sprintf($msg, $link),
+            ['name' => 'inheritance_hint']
         );
 
         $this->addElementsToGroup(
-            array('inheritance_hint'),
+            ['inheritance_hint'],
             'custom_fields',
             50,
             $this->translate('Custom properties')
         );
 
-        $this->setSubmitLabel(
-            $this->translate('Override vars')
-        );
+        $this->setSubmitLabel($this->translate('Override vars'));
     }
 
     protected function addAssignmentElements()
@@ -257,6 +266,7 @@ class IcingaServiceForm extends DirectorObjectForm
         }
 
         $this->addNameElement()
+             ->addChoices('service')
              ->addDisabledElement()
              ->groupMainProperties()
              ->addCheckCommandElements()
@@ -272,6 +282,10 @@ class IcingaServiceForm extends DirectorObjectForm
         }
     }
 
+    /**
+     * @param IcingaHost $host
+     * @return $this
+     */
     public function setHost(IcingaHost $host)
     {
         $this->host = $host;
@@ -327,6 +341,15 @@ class IcingaServiceForm extends DirectorObjectForm
             )
         ));
 
+        if ($this->object()->isApplyRule()) {
+            $rName = 'director/service/apply/filter-by-name';
+            foreach ($this->getAuth()->getRestrictions($rName) as $restriction) {
+                $this->getElement('object_name')->addValidator(
+                    new NamePattern($restriction)
+                );
+            }
+        }
+
         return $this;
     }
 
