@@ -3,7 +3,6 @@
 namespace Icinga\Module\Director\Forms;
 
 use Icinga\Module\Director\Objects\IcingaHost;
-use Icinga\Module\Director\Objects\IcingaServiceSet;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 
 class IcingaServiceSetForm extends DirectorObjectForm
@@ -20,29 +19,7 @@ class IcingaServiceSetForm extends DirectorObjectForm
             $this->setupHost();
         }
 
-        $this->setupFields()
-             ->setButtons();
-    }
-
-    protected function setupFields()
-    {
-        /** @var IcingaServiceSet $object */
-        $object = $this->object();
-
-        $this->assertResolvedImports();
-
-        if ($this->hasBeenSent() && $services = $this->getSentValue('service')) {
-            $object->service = $services;
-        }
-
-        // TODO: disabled for now. Sets have no fields, so somehow the resolver
-        //       fails here
-        if (false && $this->assertResolvedImports()) {
-            $this->fieldLoader($object)
-                ->loadFieldsForMultipleObjects($object->getServiceObjects());
-        }
-
-        return $this;
+        $this->setButtons();
     }
 
     protected function setupTemplate()
@@ -86,9 +63,7 @@ class IcingaServiceSetForm extends DirectorObjectForm
 
         if (count($object->get('imports'))) {
             $this->addHtmlHint(
-                $this->getView()->escape(
-                    $object->getResolvedProperty('description')
-                )
+                $object->getResolvedProperty('description')
             );
         }
 
@@ -135,15 +110,15 @@ class IcingaServiceSetForm extends DirectorObjectForm
 
     protected function addAssignmentElements()
     {
-        $this->addAssignFilter(array(
-            'columns' => IcingaHost::enumProperties($this->db, 'host.'),
+        $this->addAssignFilter([
+            'suggestionContext' => 'HostFilterColumns',
             'description' => $this->translate(
                 'This allows you to configure an assignment filter. Please feel'
                 . ' free to combine as many nested operators as you want. You'
                 . ' might also want to skip this, define it later and/or just'
                 . ' add this set of services to single hosts'
             )
-        ));
+        ]);
 
         return $this;
     }
