@@ -9,9 +9,11 @@ use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Db\Migrations;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Module\Director\KickstartHelper;
-use Icinga\Module\Director\Web\Form\QuickForm;
+use Icinga\Module\Director\Web\Form\DirectorForm;
+use dipl\Html\Html;
+use dipl\Html\Link;
 
-class KickstartForm extends QuickForm
+class KickstartForm extends DirectorForm
 {
     private $config;
 
@@ -59,16 +61,17 @@ class KickstartForm extends QuickForm
         }
 
         if (! $this->endpoint && $this->getDb()->hasDeploymentEndpoint()) {
-            $hint = sprintf($this->translate(
-                'Your database looks good, you are ready to %s'
-            ), $this->getView()->qlink(
-                'start working with the Icinga Director',
-                'director',
-                null,
-                array('data-base-target' => '_main')
-            ));
+            $hint = Html::sprintf(
+                $this->translate('Your database looks good, you are ready to %s'),
+                Link::create(
+                    'start working with the Icinga Director',
+                    'director',
+                    null,
+                    ['data-base-target' => '_main']
+                )
+            );
 
-            $this->addHtmlHint($hint, array('name' => 'HINT_ready'));
+            $this->addHtmlHint($hint, ['name' => 'HINT_ready']);
             $this->getDisplayGroup('config')->addElements(
                 array($this->getElement('HINT_ready'))
             );
@@ -78,15 +81,30 @@ class KickstartForm extends QuickForm
 
         $this->addResourceDisplayGroup();
 
-        $this->addHtmlHint(
-            $this->translate(
-                'Your installation of Icinga Director has not yet been prepared for'
-                . ' deployments. This kickstart wizard will assist you with setting'
-                . ' up the connection to your Icinga 2 server.'
-            ),
-            array('name' => 'HINT_kickstart')
-            // http://docs.icinga.com/icinga2/latest/doc/module/icinga2/chapter/object-types#objecttype-apilistener
-        );
+        if ($this->getDb()->hasDeploymentEndpoint()) {
+            $this->addHtmlHint(
+                $this->translate(
+                    'Your configuration looks good. Still, you might want to re-run'
+                    . ' this kickstart wizard to (re-)import modified or new manually'
+                    . ' defined Command definitions or to get fresh new ITL commands'
+                    . ' after an Icinga 2 Core upgrade.'
+                ),
+                array('name' => 'HINT_kickstart')
+                // http://docs.icinga.com/icinga2/latest/doc/module/icinga2/chapter/
+                // ... object-types#objecttype-apilistener
+            );
+        } else {
+            $this->addHtmlHint(
+                $this->translate(
+                    'Your installation of Icinga Director has not yet been prepared for'
+                    . ' deployments. This kickstart wizard will assist you with setting'
+                    . ' up the connection to your Icinga 2 server.'
+                ),
+                array('name' => 'HINT_kickstart')
+                // http://docs.icinga.com/icinga2/latest/doc/module/icinga2/chapter/
+                // ... object-types#objecttype-apilistener
+            );
+        }
 
         $this->addElement('text', 'endpoint', array(
             'label'       => $this->translate('Endpoint Name'),
@@ -169,7 +187,7 @@ class KickstartForm extends QuickForm
         if ($resourceName = $this->getResourceName()) {
             $resourceConfig = ResourceFactory::getResourceConfig($resourceName);
             if (! isset($resourceConfig->charset)
-                || $resourceConfig->charset !== 'utf8'
+                || ! in_array($resourceConfig->charset, array('utf8', 'utf8mb4'))
             ) {
                 $this->getElement('resource')
                     ->addError('Please change the encoding for the director database to utf8');
@@ -219,14 +237,15 @@ class KickstartForm extends QuickForm
                 $this->translate('This has to be a MySQL or PostgreSQL database')
             );
 
-            $hint = $this->translate('Please click %s to create new DB resources');
-            $link = $this->getView()->qlink(
-                $this->translate('here'),
-                'config/resource',
-                null,
-                array('data-base-target' => '_main')
-            );
-            $this->addHtmlHint(sprintf($hint, $link));
+            $this->addHtmlHint(Html::sprintf(
+                $this->translate('Please click %s to create new DB resources'),
+                Link::create(
+                    $this->translate('here'),
+                    'config/resource',
+                    null,
+                    ['data-base-target' => '_main']
+                )
+            ));
         }
 
         $this->setSubmitLabel($this->storeConfigLabel);
@@ -294,7 +313,7 @@ class KickstartForm extends QuickForm
                 )
             );
             $this->addHtmlHint(
-                '<pre>' . $config . '</pre>',
+                Html::tag('pre', null, $config),
                 array('name' => 'HINT_config_store')
             );
 
@@ -315,33 +334,30 @@ class KickstartForm extends QuickForm
 
     public function onSuccess()
     {
-        try {
-            if ($this->getSubmitLabel() === $this->storeConfigLabel) {
-                if ($this->storeResourceConfig()) {
-                    parent::onSuccess();
-                } else {
-                    return;
-                }
-            }
-
-            if ($this->getSubmitLabel() === $this->createDbLabel
-                || $this->getSubmitLabel() === $this->migrateDbLabel) {
-                $this->migrations()->applyPendingMigrations();
+        if ($this->getSubmitLabel() === $this->storeConfigLabel) {
+            if ($this->storeResourceConfig()) {
                 parent::onSuccess();
+            } else {
+                return;
             }
-
-            $values = $this->getValues();
-            if ($this->endpoint && empty($values['password'])) {
-                $values['password'] = $this->endpoint->getApiUser()->password;
-            }
-
-            $kickstart = new KickstartHelper($this->getDb());
-            unset($values['resource']);
-            $kickstart->setConfig($values)->run();
-            parent::onSuccess();
-        } catch (Exception $e) {
-            $this->addError($e->getMessage());
         }
+
+        if ($this->getSubmitLabel() === $this->createDbLabel
+            || $this->getSubmitLabel() === $this->migrateDbLabel) {
+            $this->migrations()->applyPendingMigrations();
+            parent::onSuccess();
+        }
+
+        $values = $this->getValues();
+        if ($this->endpoint && empty($values['password'])) {
+            $values['password'] = $this->endpoint->getApiUser()->password;
+        }
+
+        $kickstart = new KickstartHelper($this->getDb());
+        unset($values['resource']);
+        $kickstart->setConfig($values)->run();
+
+        parent::onSuccess();
     }
 
     protected function getResourceName()
@@ -359,7 +375,7 @@ class KickstartForm extends QuickForm
         }
     }
 
-    protected function getDb()
+    public function getDb()
     {
         return Db::fromResourceName($this->getResourceName());
     }
