@@ -1,30 +1,18 @@
 <?php
 
-// TODO: Check whether this can be removed
 namespace Icinga\Module\Director\Forms;
 
-use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Objects\IcingaObject;
-use Icinga\Module\Director\Web\Form\QuickForm;
+use Icinga\Module\Director\Web\Form\DirectorForm;
 
-class RestoreObjectForm extends QuickForm
+class RestoreObjectForm extends DirectorForm
 {
-    /** @var Db */
-    protected $db;
-
     /** @var IcingaObject */
     protected $object;
 
     public function setup()
     {
-        $this->submitLabel = $this->translate('Restore former object');
-    }
-
-    protected function addSubmitButtonIfSet()
-    {
-        $res = parent::addSubmitButtonIfSet();
-        $this->getDisplayGroup('buttons')->setDecorators(array('FormElements'));
-        return $res;
+        $this->addSubmitButton($this->translate('Restore former object'));
     }
 
     public function onSuccess()
@@ -32,24 +20,27 @@ class RestoreObjectForm extends QuickForm
         $object = $this->object;
         $name = $object->getObjectName();
         $db = $this->db;
-        $msg = $this->translate('Object has been restored');
 
         // TODO: service -> multi-key
         if ($object::exists($name, $db)) {
-            $object::load($name, $db)->replaceWith($object)->store();
+            $existing = $object::load($name, $db)->replaceWith($object);
+
+            if ($existing->hasBeenModified()) {
+                $msg = $this->translate('Object has been restored');
+                $existing->store();
+            } else {
+                $msg = $this->translate(
+                    'Nothing to do, restore would not modify the current object'
+                );
+            }
         } else {
+            $msg = $this->translate('Object has been re-created');
             $object->store($db);
         }
 
         $this->redirectOnSuccess($msg);
     }
 
-    public function setDb($db)
-    {
-        $this->db = $db;
-        return $this;
-    }
-
     public function setObject(IcingaObject $object)
     {
         $this->object = $object;
