@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Forms;
 
 use Exception;
 use Icinga\Module\Director\Hook\ImportSourceHook;
+use Icinga\Module\Director\Objects\SyncProperty;
 use Icinga\Module\Director\Objects\SyncRule;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Objects\ImportSource;
@@ -288,24 +289,30 @@ class SyncPropertyForm extends DirectorObjectForm
         }
 
         foreach ($dummy->listProperties() as $prop) {
-            if ($prop === 'id') {
+            if ($dummy instanceof IcingaObject && $prop === 'id') {
                 continue;
             }
 
             // TODO: allow those fields, but munge them (store ids)
             //if (preg_match('~_id$~', $prop)) continue;
             if (substr($prop, -3) === '_id') {
-                $prop = substr($prop, 0, -3);
-                if (! $dummy instanceof IcingaObject || ! $dummy->hasRelation($prop)) {
-                    continue;
+                $short = substr($prop, 0, -3);
+                if ($dummy instanceof IcingaObject) {
+                    if ($dummy->hasRelation($short)) {
+                        $prop = $short;
+                    } else {
+                        continue;
+                    }
                 }
             }
 
             $props[$prop] = $prop;
         }
 
-        foreach ($dummy->listMultiRelations() as $prop) {
-            $props[$prop] = sprintf('%s (%s)', $prop, $this->translate('a list'));
+        if ($dummy instanceof IcingaObject) {
+            foreach ($dummy->listMultiRelations() as $prop) {
+                $props[$prop] = sprintf('%s (%s)', $prop, $this->translate('a list'));
+            }
         }
 
         ksort($props);
@@ -349,6 +356,7 @@ class SyncPropertyForm extends DirectorObjectForm
 
     public function onSuccess()
     {
+        /** @var SyncProperty $object */
         $object = $this->getObject();
         $object->set('rule_id', $this->rule->get('id')); // ?!
 
@@ -357,7 +365,6 @@ class SyncPropertyForm extends DirectorObjectForm
         }
 
         $sourceColumn = $this->getValue('source_column');
-        unset($this->source_column);
         $this->removeElement('source_column');
 
         if ($sourceColumn !== self::EXPRESSION) {
@@ -370,13 +377,6 @@ class SyncPropertyForm extends DirectorObjectForm
             $object->destination_field = 'vars.' . $destination;
         }
 
-        if ($object->hasBeenModified()) {
-            if (! $object->hasBeenLoadedFromDb()) {
-                $object->priority = $this->rule->getPriorityForNextProperty();
-            }
-        }
-
-
         return parent::onSuccess();
     }
 
