@@ -1,12 +1,11 @@
 <?php
 
-// TODO: Check whether this can be removed
 namespace Icinga\Module\Director\Forms;
 
 use Icinga\Module\Director\Objects\SyncRule;
-use Icinga\Module\Director\Web\Form\QuickForm;
+use Icinga\Module\Director\Web\Form\DirectorForm;
 
-class SyncRunForm extends QuickForm
+class SyncRunForm extends DirectorForm
 {
     /** @var SyncRule */
     protected $rule;
