@@ -3,6 +3,7 @@
 namespace Icinga\Module\Director\Cli;
 
 use Icinga\Cli\Command as CliCommand;
+use Icinga\Module\Director\Application\MemoryLimit;
 use Icinga\Module\Director\Core\CoreApi;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
@@ -73,6 +74,25 @@ class Command extends CliCommand
         return $this->api;
     }
 
+    /**
+     * Raise PHP resource limits
+     *
+     * TODO: do this in a failsafe way, and only if necessary
+     *
+     * @return self;
+     */
+    protected function raiseLimits()
+    {
+        MemoryLimit::raiseTo('1024M');
+
+        ini_set('max_execution_time', 0);
+        if (version_compare(PHP_VERSION, '7.0.0') < 0) {
+            ini_set('zend.enable_gc', 0);
+        }
+
+        return $this;
+    }
+
     /**
      * @return Db
      */
