@@ -23,6 +23,8 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
 
     private $position = 0;
 
+    private $overrideKeyName;
+
     protected $idx = array();
 
     protected static $allTables = array(
@@ -61,6 +63,15 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
         }
     }
 
+    public static function renameAll($oldname, $newname, Db $connection)
+    {
+        $db = $connection->getDbAdapter();
+        $where = $db->quoteInto('varname = ?', $oldname);
+        foreach (static::$allTables as $table) {
+            $db->update($table, ['varname' => $newname], $where);
+        }
+    }
+
     public function count()
     {
         $count = 0;
@@ -102,7 +113,6 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
         return array_key_exists($this->position, $this->idx);
     }
 
-
     /**
      * Generic setter
      *
@@ -293,6 +303,12 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
         return sha1(implode('|', $sums), true);
     }
 
+    public function setOverrideKeyName($name)
+    {
+        $this->overrideKeyName = $name;
+        return $this;
+    }
+
     public function toConfigString($renderExpressions = false)
     {
         $out = '';
@@ -351,15 +367,23 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
      */
     protected function renderSingleVar($key, $var, $renderExpressions = false)
     {
-        return c::renderKeyValue(
-            $this->renderKeyName($key),
-            $var->toConfigStringPrefetchable($renderExpressions)
-        );
+        if ($key === $this->overrideKeyName) {
+            return c::renderKeyOperatorValue(
+                $this->renderKeyName($key),
+                '+=',
+                $var->toConfigStringPrefetchable($renderExpressions)
+            );
+        } else {
+            return c::renderKeyValue(
+                $this->renderKeyName($key),
+                $var->toConfigStringPrefetchable($renderExpressions)
+            );
+        }
     }
 
     protected function renderKeyName($key)
     {
-        if (preg_match('/^[a-z0-9_]+\d*$/i', $key)) {
+        if (preg_match('/^[a-z][a-z0-9_]*$/i', $key)) {
             return 'vars.' . c::escapeIfReserved($key);
         } else {
             return 'vars[' . c::renderString($key) . ']';
