@@ -2,13 +2,17 @@
 
 namespace Icinga\Module\Director\Dashboard;
 
+use Icinga\Module\Director\Web\Tabs\InfraTabs;
+
 class InfrastructureDashboard extends Dashboard
 {
     protected $dashletNames = array(
         'Kickstart',
+        'SelfService',
         'ApiUserObject',
         'EndpointObject',
         'ZoneObject',
+        'Settings',
     );
 
     public function getTitle()
@@ -29,4 +33,9 @@ class InfrastructureDashboard extends Dashboard
             . ' clean up the whole mess afterwards.'
         );
     }
+
+    public function getTabs()
+    {
+        return new InfraTabs($this->getAuth());
+    }
 }
