@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Data\Db;
 
 use Icinga\Exception\IcingaException as IE;
 use Icinga\Exception\NotFoundError;
+use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\Util;
 use Exception;
@@ -243,16 +244,32 @@ abstract class DbObject
             return $this->$func();
         }
 
-        if (! array_key_exists($property, $this->properties)) {
-            throw new IE('Trying to get invalid property "%s"', $property);
-        }
+        $this->assertPropertyExists($property);
         return $this->properties[$property];
     }
 
+    public function getProperty($key)
+    {
+        $this->assertPropertyExists($key);
+        return $this->properties[$key];
+    }
+
+    protected function assertPropertyExists($key)
+    {
+        if (! array_key_exists($key, $this->properties)) {
+            throw new IE('Trying to get invalid property "%s"', $key);
+        }
+
+        return $this;
+    }
+
     public function hasProperty($key)
     {
         if (array_key_exists($key, $this->properties)) {
             return true;
+        } elseif ($key === 'id') {
+            // There is getId, would give false positive
+            return false;
         }
         $func = 'get' . ucfirst($key);
         if (substr($func, -2) === '[]') {
@@ -441,6 +458,16 @@ abstract class DbObject
         return $props;
     }
 
+    /**
+     * List all properties that changed since object creation
+     *
+     * @return array
+     */
+    public function listModifiedProperties()
+    {
+        return array_keys($this->modifiedProperties);
+    }
+
     /**
      * Whether this object has been modified
      *
@@ -527,12 +554,12 @@ abstract class DbObject
     /**
      * Get the autoinc value if set
      *
-     * @return string
+     * @return int
      */
     public function getAutoincId()
     {
         if (isset($this->properties[$this->autoincKeyName])) {
-            return $this->properties[$this->autoincKeyName];
+            return (int) $this->properties[$this->autoincKeyName];
         }
         return null;
     }
@@ -597,6 +624,18 @@ abstract class DbObject
         return $this->setDbProperties($properties);
     }
 
+    /**
+     * @param object $row
+     * @param Db $db
+     * @return self
+     */
+    public static function fromDbRow($row, Db $db)
+    {
+        return (new static())
+            ->setConnection($db)
+            ->setDbProperties($row);
+    }
+
     protected function setDbProperties($properties)
     {
         foreach ($properties as $key => $val) {
@@ -629,6 +668,26 @@ abstract class DbObject
         return $this->loadedProperties;
     }
 
+    public function getOriginalProperty($key)
+    {
+        $this->assertPropertyExists($key);
+        if ($this->hasBeenLoadedFromDb()) {
+            return $this->loadedProperties[$key];
+        }
+
+        return null;
+    }
+
+    public function resetProperty($key)
+    {
+        $this->set($key, $this->getOriginalProperty($key));
+        if ($this->listModifiedProperties() === [$key]) {
+            $this->hasBeenModified = false;
+        }
+
+        return $this;
+    }
+
     public function hasBeenLoadedFromDb()
     {
         return $this->loadedFromDb;
@@ -818,7 +877,7 @@ abstract class DbObject
         return $result !== false;
     }
 
-    protected function createWhere()
+    public function createWhere()
     {
         if ($id = $this->getAutoincId()) {
             return $this->db->quoteInto(
@@ -1011,6 +1070,14 @@ abstract class DbObject
 
     public static function loadWithAutoIncId($id, DbConnection $connection)
     {
+        /* Need to cast to int, otherwise the id will be matched against
+         * object_name, which may wreak havoc if an object has a
+         * object_name matching some id. Note that DbObject::set() and
+         * DbObject::setDbProperties() will convert any property to
+         * string, including ids.
+         */
+        $id = (int) $id;
+
         if ($prefetched = static::getPrefetched($id)) {
             return $prefetched;
         }
@@ -1040,7 +1107,7 @@ abstract class DbObject
      * @param \Zend_Db_Select $query
      * @param string|null $keyColumn
      *
-     * @return self[]
+     * @return static[]
      */
     public static function loadAll(DbConnection $connection, $query = null, $keyColumn = null)
     {
