@@ -3,7 +3,6 @@
 namespace Icinga\Module\Director;
 
 use Icinga\Module\Director\Data\Db\DbConnection;
-use Icinga\Module\Director\Objects\DirectorDeploymentLog;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Exception\ConfigurationError;
@@ -390,6 +389,7 @@ class Db extends DbConnection
             'apiuser',
             'endpoint',
             'zone',
+            'dependency',
         );
 
         $queries = array();
@@ -591,26 +591,6 @@ class Db extends DbConnection
         return $this->enum('icinga_' . $type, null, $filters);
     }
 
-    public function listExternal($type)
-    {
-        $table = IcingaObject::createByType($type)->getTableName();
-
-        $select = $this->db()->select()->from(
-            array('o' => $table),
-            array('object_name' => 'o.object_name')
-        )->where(
-            'object_type = ?',
-            'external_object'
-        )->order('o.object_name');
-
-        $res = $this->db()->fetchCol($select);
-        if (empty($res)) {
-            return array();
-        }
-
-        return array_combine($res, $res);
-    }
-
     public function fetchDistinctHostVars()
     {
         $select = $this->db()->select()->distinct()->from(
@@ -692,33 +672,4 @@ class Db extends DbConnection
 
         return $db->fetchPairs($query);
     }
-
-    /**
-     * @return DirectorDeploymentLog[]
-     */
-    public function getUncollectedDeployments()
-    {
-        $db = $this->db();
-
-        $query = $db->select()
-            ->from('director_deployment_log')
-            ->where('stage_name IS NOT NULL')
-            ->where('stage_collected IS NULL')
-            ->where('startup_succeeded IS NULL')
-            ->order('stage_name');
-
-        return DirectorDeploymentLog::loadAll($this, $query, 'stage_name');
-    }
-
-    public function hasUncollectedDeployments()
-    {
-        $db = $this->db();
-        $query = $db->select()
-            ->from('director_deployment_log', array('cnt' => 'COUNT(*)'))
-            ->where('stage_name IS NOT NULL')
-            ->where('stage_collected IS NULL')
-            ->where('startup_succeeded IS NULL');
-
-        return $db->fetchOne($query) > 0;
-    }
 }
