@@ -9,6 +9,7 @@ use Icinga\Exception\ConfigurationError;
 use Icinga\Exception\IcingaException;
 use Icinga\Exception\NotFoundError;
 use Icinga\Exception\ProgrammingError;
+use Icinga\Module\Director\Application\MemoryLimit;
 use Icinga\Module\Director\Db\Cache\PrefetchCache;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Hook\ShipConfigFilesHook;
@@ -161,7 +162,7 @@ class IcingaConfig
     }
 
     /**
-     * @return string
+     * @return array
      */
     public function getFileNames()
     {
@@ -447,11 +448,7 @@ class IcingaConfig
 
         $start = microtime(true);
 
-        // Raise limits. TODO: do this in a failsafe way, and only if necessary
-        if ((string) ini_get('memory_limit') !== '-1') {
-            ini_set('memory_limit', '1024M');
-        }
-
+        MemoryLimit::raiseTo('1024M');
         ini_set('max_execution_time', 0);
         // Workaround for https://bugs.php.net/bug.php?id=68606 or similar
         ini_set('zend.enable_gc', 0);
@@ -477,6 +474,7 @@ class IcingaConfig
             ->createFileFromDb('userGroup')
             ->createFileFromDb('user')
             ->createFileFromDb('notification')
+            ->createFileFromDb('dependency')
             ;
 
         if (! $this->isLegacy()) {
@@ -719,7 +717,8 @@ apply Service for (title => params in host.vars["%s"]) {
             'user',
             'userGroup',
             'timePeriod',
-            'notification'
+            'notification',
+            'dependency'
         );
 
         return in_array($type, $types);
