@@ -2,6 +2,7 @@
 
 namespace Icinga\Module\Director\IcingaConfig;
 
+use Icinga\Exception\IcingaException;
 use Icinga\Exception\ProgrammingError;
 
 class IcingaConfigHelper
@@ -63,15 +64,36 @@ class IcingaConfigHelper
 
     public static function renderBoolean($value)
     {
-        if ($value === 'y') {
+        if ($value === 'y' || $value === true) {
             return 'true';
-        } elseif ($value === 'n') {
+        } elseif ($value === 'n' || $value === false) {
             return 'false';
         } else {
             throw new ProgrammingError('%s is not a valid boolean', $value);
         }
     }
 
+    protected static function renderInteger($value)
+    {
+        return (string) $value;
+    }
+
+    protected static function renderFloat($value)
+    {
+        // Render .0000 floats as integers, mainly because of some JSON
+        // implementations:
+        if ((string) (int) $value === (string) $value) {
+            return static::renderInteger((int) $value);
+        } else {
+            return sprintf('%F', $value);
+        }
+    }
+
+    protected static function renderNull()
+    {
+        return 'null';
+    }
+
     // TODO: Find out how to allow multiline {{{...}}} strings.
     //       Parameter? Dedicated method? Always if \n is found?
     public static function renderString($string)
@@ -103,6 +125,29 @@ class IcingaConfigHelper
         return '"' . $string . '"';
     }
 
+    public static function renderPhpValue($value)
+    {
+        if (is_null($value)) {
+            return static::renderNull();
+        } elseif (is_bool($value)) {
+            return static::renderBoolean($value);
+        } elseif (is_integer($value)) {
+            return static::renderInteger($value);
+        } elseif (is_float($value)) {
+            return static::renderFloat($value);
+        // TODO:
+        // } elseif (is_object($value) || static::isAssocArray($value)) {
+        //     return static::renderHash($value, $prefix)
+        // TODO: also check array
+        } elseif (is_array($value)) {
+            return static::renderArray($value);
+        } elseif (is_string($value)) {
+            return static::renderString($value);
+        } else {
+            throw new IcingaException('Unexpected type %s', var_export($value, 1));
+        }
+    }
+
     public static function renderDictionaryKey($key)
     {
         if (preg_match('/^[a-z_]+[a-z0-9_]*$/i', $key)) {
@@ -273,6 +318,17 @@ class IcingaConfigHelper
             static::renderString($string)
         );
 
+        // TODO: this is an exemption for special variables. It would
+        //       never make any sense to evaluate them at parse time.
+        //       Another issue remains: there might be other reasons
+        //       for "late evaluation". But how to distinguish those
+        //       use cases?
+        $string = preg_replace(
+            '/" \+ ((?:user|notification)\.[\w\.]+) \+ "/',
+            '\$${1}\$',
+            $string
+        );
+
         if (substr($string, 0, 5) === '"" + ') {
             $string = substr($string, 5);
         }
