@@ -4,11 +4,17 @@ namespace Icinga\Module\Director\Import;
 
 use Exception;
 use Icinga\Data\Filter\Filter;
+use Icinga\Module\Director\Application\MemoryLimit;
+use Icinga\Module\Director\Data\Db\DbObject;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Db\Cache\PrefetchCache;
+use Icinga\Module\Director\Objects\HostGroupMembershipResolver;
+use Icinga\Module\Director\Objects\IcingaHost;
+use Icinga\Module\Director\Objects\IcingaHostGroup;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Objects\ImportSource;
 use Icinga\Module\Director\Objects\IcingaService;
+use Icinga\Module\Director\Objects\SyncProperty;
 use Icinga\Module\Director\Objects\SyncRule;
 use Icinga\Module\Director\Objects\SyncRun;
 use Icinga\Module\Director\Util;
@@ -48,7 +54,7 @@ class Sync
     /**
      * Objects to work with
      *
-     * @var array
+     * @var IcingaObject[]
      */
     protected $objects;
 
@@ -67,8 +73,11 @@ class Sync
 
     protected $errors = array();
 
+    /** @var SyncProperty[] */
     protected $syncProperties;
 
+    protected $replaceVars = false;
+
     /**
      * @var SyncRun
      */
@@ -76,8 +85,12 @@ class Sync
 
     protected $runStartTime;
 
+    /** @var Filter[] */
     protected $columnFilters = array();
 
+    /** @var HostGroupMembershipResolver|bool */
+    protected $hostGroupMembershipResolver;
+
     /**
      * Constructor. No direct initialization allowed right now. Please use one
      * of the available static factory methods
@@ -141,13 +154,11 @@ class Sync
     /**
      * Raise PHP resource limits
      *
-     * TODO: do this in a failsafe way, and only if necessary
-     *
      * @return self;
      */
     protected function raiseLimits()
     {
-        ini_set('memory_limit', '768M');
+        MemoryLimit::raiseTo('1024M');
         ini_set('max_execution_time', 0);
 
         return $this;
@@ -172,8 +183,12 @@ class Sync
      */
     protected function fetchSyncProperties()
     {
-        $this->syncProperties = $this->rule->fetchSyncProperties();
+        $this->syncProperties = $this->rule->getSyncProperties();
         foreach ($this->syncProperties as $key => $prop) {
+            if ($prop->destination_field === 'vars' && $prop->merge_policy === 'override') {
+                $this->replaceVars = true;
+            }
+
             if (! strlen($prop->filter_expression)) {
                 continue;
             }
@@ -240,8 +255,8 @@ class Sync
 
     /**
      * Fetch latest imported data rows from all involved import sources
-     *
-     * @return self
+     * @return Sync
+     * @throws IcingaException
      */
     protected function fetchImportedData()
     {
@@ -258,9 +273,28 @@ class Sync
             $key = $source->key_column;
             $this->sourceColumns[$sourceId][$key] = $key;
             $run = $source->fetchLastRun(true);
-            $rows = $run->fetchRows(
-                SyncUtils::getRootVariables($this->sourceColumns[$sourceId])
-            );
+
+            $usedColumns = SyncUtils::getRootVariables($this->sourceColumns[$sourceId]);
+
+            $filterColumns = array();
+            foreach ($this->columnFilters as $filter) {
+                foreach ($filter->listFilteredColumns() as $column) {
+                    $filterColumns[$column] = $column;
+                }
+            }
+            if (($ruleFilter = $this->rule->filter()) !== null) {
+                foreach ($ruleFilter->listFilteredColumns() as $column) {
+                    $filterColumns[$column] = $column;
+                }
+            }
+
+            if (! empty($filterColumns)) {
+                foreach (SyncUtils::getRootVariables($filterColumns) as $column) {
+                    $usedColumns[$column] = $column;
+                }
+            }
+
+            $rows = $run->fetchRows($usedColumns);
 
             $this->imported[$sourceId] = array();
             foreach ($rows as $row) {
@@ -375,7 +409,7 @@ class Sync
 
         // TODO: should be obsoleted by a better "loadFiltered" method
         if ($this->rule->object_type === 'datalistEntry') {
-            $this->removeForeignListEntries($this->objects);
+            $this->removeForeignListEntries();
         }
 
         return $this;
@@ -416,7 +450,11 @@ class Sync
                         $newVars[$varName] = $val;
                     } else {
                         if ($prop === 'import') {
-                            $imports[] = $val;
+                            if (is_array($val)) {
+                                $imports = array_merge($imports, $val);
+                            } elseif (!is_null($val)) {
+                                $imports[] = $val;
+                            }
                         } else {
                             $newProps[$prop] = $val;
                         }
@@ -466,12 +504,68 @@ class Sync
         return $newObjects;
     }
 
+    protected function deferResolvers()
+    {
+        if (in_array($this->rule->get('object_type'), array('host', 'hostgroup'))) {
+            $resolver = $this->getHostGroupMembershipResolver();
+            $resolver->defer()->setUseTransactions(false);
+        }
+
+        return $this;
+    }
+
+    /**
+     * @param DbObject $object
+     * @return $this
+     */
+    protected function setResolver($object)
+    {
+        if (! ($object instanceof IcingaHost || $object instanceof IcingaHostGroup)) {
+            return $this;
+        }
+        if ($resolver = $this->getHostGroupMembershipResolver()) {
+            $object->setHostGroupMembershipResolver($resolver);
+        }
+
+        return $this;
+    }
+
+    protected function notifyResolvers()
+    {
+        if ($resolver = $this->getHostGroupMembershipResolver()) {
+            $resolver->refreshDb(true);
+        }
+
+        return $this;
+    }
+
+    /**
+     * @return bool|HostGroupMembershipResolver
+     */
+    protected function getHostGroupMembershipResolver()
+    {
+        if ($this->hostGroupMembershipResolver === null) {
+            if (in_array(
+                $this->rule->get('object_type'),
+                array('host', 'hostgroup')
+            )) {
+                $this->hostGroupMembershipResolver = new HostGroupMembershipResolver(
+                    $this->db
+                );
+            } else {
+                $this->hostGroupMembershipResolver = false;
+            }
+        }
+
+        return $this->hostGroupMembershipResolver;
+    }
+
     /**
      * Evaluates a SyncRule and returns a list of modified objects
      *
      * TODO: This needs to be splitted into smaller methods
      *
-     * @return array          List of modified IcingaObjects
+     * @return DbObject[]          List of modified IcingaObjects
      */
     protected function prepare()
     {
@@ -487,14 +581,23 @@ class Sync
              ->prepareRelatedImportSources()
              ->prepareSourceColumns()
              ->loadExistingObjects()
-             ->fetchImportedData();
+             ->fetchImportedData()
+             ->deferResolvers();
 
         // TODO: directly work on existing objects, remember imported keys, then purge
         $newObjects = $this->prepareNewObjects();
 
+        $hasDisabled = false;
+        foreach ($this->syncProperties as $property) {
+            if ($property->get('destination_field') === 'disabled') {
+                $hasDisabled = true;
+            }
+        }
+
+        /** @var DbObject|IcingaObject $object */
         foreach ($newObjects as $key => $object) {
             if (array_key_exists($key, $this->objects)) {
-                switch ($this->rule->update_policy) {
+                switch ($this->rule->get('update_policy')) {
                     case 'override':
                         $this->objects[$key]->replaceWith($object);
                         break;
@@ -502,7 +605,10 @@ class Sync
                     case 'merge':
                         // TODO: re-evaluate merge settings. vars.x instead of
                         //       just "vars" might suffice.
-                        $this->objects[$key]->merge($object);
+                        $this->objects[$key]->merge($object, $this->replaceVars);
+                        if (! $hasDisabled && $object->hasProperty('disabled')) {
+                            $this->objects[$key]->resetProperty('disabled');
+                        }
                         break;
 
                     default:
@@ -545,8 +651,9 @@ class Sync
 
     /**
      * Runs a SyncRule and applies all resulting changes
-     *
      * @return int
+     * @throws Exception
+     * @throws IcingaException
      */
     public function apply()
     {
@@ -565,8 +672,9 @@ class Sync
             $modified = 0;
             $deleted = 0;
             foreach ($objects as $object) {
+                $this->setResolver($object);
                 if ($object->shouldBeRemoved()) {
-                    $object->delete($db);
+                    $object->delete();
                     $deleted++;
                     continue;
                 }
@@ -596,7 +704,7 @@ class Sync
             }
 
             $this->run->setProperties($runProperties)->store();
-
+            $this->notifyResolvers();
             $dba->commit();
 
             // Store duration after commit, as the commit might take some time
