@@ -18,6 +18,7 @@ use Icinga\Module\Director\IcingaConfig\IcingaConfig;
 use Icinga\Module\Director\IcingaConfig\IcingaConfigRenderer;
 use Icinga\Module\Director\IcingaConfig\IcingaConfigHelper as c;
 use Icinga\Module\Director\IcingaConfig\IcingaLegacyConfigHelper as c1;
+use Icinga\Module\Director\Repository\IcingaTemplateRepository;
 
 abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 {
@@ -36,9 +37,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     /** @var bool Whether this Object makes use of (time) ranges */
     protected $supportsRanges = false;
 
-    /** @var bool Whether this object supports (command) Arguments */
-    protected $supportsArguments = false;
-
     /** @var bool Whether inheritance via "imports" property is supported */
     protected $supportsImports = false;
 
@@ -51,6 +49,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     /** @var bool Whether Sets of object can be defined */
     protected $supportsSets = false;
 
+    /** @var bool Whether this Object supports template-based Choices */
+    protected $supportsChoices = false;
+
     /** @var bool If the object is rendered in legacy config */
     protected $supportedInLegacy = false;
 
@@ -117,8 +118,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     /** @var  IcingaTimePeriodRanges - TODO: generic ranges */
     private $ranges;
 
-    private $arguments;
-
     private $shouldBeRemoved = false;
 
     private $resolveCache = array();
@@ -127,6 +126,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     private $templateResolver;
 
+    protected static $tree;
+
     /**
      * @return Db
      */
@@ -318,6 +319,24 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return null;
     }
 
+    public function prefetchAllRelatedTypes()
+    {
+        foreach (array_unique(array_values($this->relations)) as $relClass) {
+            /** @var static $class */
+            $class = __NAMESPACE__ . '\\' . $relClass;
+            $class::prefetchAll($this->getConnection());
+        }
+    }
+
+    public static function prefetchAllRelationsByType($type, Db $db)
+    {
+        /** @var static $class */
+        $class = self::classByType($type);
+        /** @var static $dummy */
+        $dummy = $class::create(array(), $db);
+        $dummy->prefetchAllRelatedTypes();
+    }
+
     /**
      * Whether this Object supports custom variables
      *
@@ -355,7 +374,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      */
     public function supportsArguments()
     {
-        return $this->supportsArguments;
+        return $this instanceof ObjectWithArguments;
     }
 
     /**
@@ -408,6 +427,16 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->supportsSets;
     }
 
+    /**
+     * Whether this object supports template-based Choices
+     *
+     * @return bool
+     */
+    public function supportsChoices()
+    {
+        return $this->supportsChoices;
+    }
+
     public function setAssignments($value)
     {
         return IcingaObjectLegacyAssignments::applyToObject($this, $value);
@@ -480,7 +509,18 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     public function hasBeenModified()
     {
-        $this->resolveUnresolvedRelatedProperties();
+        if (parent::hasBeenModified()) {
+            return true;
+        }
+
+        if ($this->hasUnresolvedRelatedProperties()) {
+            $this->resolveUnresolvedRelatedProperties();
+
+            // Duplicates above code, but this makes it faster:
+            if (parent::hasBeenModified()) {
+                return true;
+            }
+        }
 
         if ($this->supportsCustomVars() && $this->vars !== null && $this->vars()->hasBeenModified()) {
             return true;
@@ -498,7 +538,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             return true;
         }
 
-        if ($this->supportsArguments() && $this->arguments !== null && $this->arguments()->hasBeenModified()) {
+        if ($this instanceof ObjectWithArguments
+            && $this->gotArguments()
+            && $this->arguments()->hasBeenModified()
+        ) {
             return true;
         }
 
@@ -514,7 +557,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
-        return parent::hasBeenModified();
+        return false;
+    }
+
+    protected function hasUnresolvedRelatedProperties()
+    {
+        return ! empty($this->unresolvedRelatedProperties);
     }
 
     protected function hasUnresolvedRelatedProperty($name)
@@ -613,7 +661,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             //TODO: allow for deep keys
             $this->vars()->set(substr($key, 5), $value);
             return $this;
-        } elseif (substr($key, 0, 10) === 'arguments.') {
+        } elseif ($this instanceof ObjectWithArguments
+            && substr($key, 0, 10) === 'arguments.'
+        ) {
             $this->arguments()->set(substr($key, 10), $value);
             return $this;
         }
@@ -674,17 +724,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this;
     }
 
-    protected function setArguments($value)
-    {
-        $this->arguments()->setArguments($value);
-        return $this;
-    }
-
-    protected function getArguments()
-    {
-        return $this->arguments()->toPlainObject();
-    }
-
     protected function getRanges()
     {
         return $this->ranges()->getValues();
@@ -727,6 +766,15 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->shouldBeRemoved;
     }
 
+    public function shouldBeRenamed()
+    {
+        return $this->hasBeenLoadedFromDb()
+            && $this->getOriginalProperty('object_name') !== $this->getObjectName();
+    }
+
+    /**
+     * @return IcingaObjectGroups[]
+     */
     public function groups()
     {
         $this->assertGroupsSupport();
@@ -769,20 +817,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this->rangeClass;
     }
 
-    public function arguments()
-    {
-        $this->assertArgumentsSupport();
-        if ($this->arguments === null) {
-            if ($this->hasBeenLoadedFromDb()) {
-                $this->arguments = IcingaArguments::loadForStoredObject($this);
-            } else {
-                $this->arguments = new IcingaArguments($this);
-            }
-        }
-
-        return $this->arguments;
-    }
-
     /**
      * @return IcingaObjectImports
      */
@@ -826,9 +860,13 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     public function getImports()
     {
-        return $this->imports()->listImportNames();
+        return $this->listImportNames();
     }
 
+    /**
+     * @deprecated This should no longer be in use
+     * @return IcingaTemplateResolver
+     */
     public function templateResolver()
     {
         if ($this->templateResolver === null) {
@@ -1036,7 +1074,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     protected function triggerLoopDetection()
     {
-        $this->templateResolver()->listResolvedParentIds();
+        // $this->templateResolver()->listResolvedParentIds();
     }
 
     public function getSingleResolvedProperty($key, $default = null)
@@ -1078,7 +1116,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         $vals['_MERGED_']    = (object) array();
         $vals['_INHERITED_'] = (object) array();
         $vals['_ORIGINS_']   = (object) array();
-        $objects = $this->imports()->getObjects();
+        // $objects = $this->imports()->getObjects();
+        $objects = IcingaTemplateRepository::instanceByObject($this)
+            ->getTemplatesIndexedByNameFor($this, true);
 
         $get          = 'get'         . $what;
         $getInherited = 'getInherited' . $what;
@@ -1093,6 +1133,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                     continue;
                 }
 
+                if (! property_exists($origins, $key)) {
+                    // TODO:  Introduced with group membership resolver or
+                    //        choices - this should not be required. Check this!
+                    continue;
+                }
+
                 // $vals[$name]->$key = $value;
                 $vals['_MERGED_']->$key = $value;
                 $vals['_INHERITED_']->$key = $value;
@@ -1171,18 +1217,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $this;
     }
 
-    protected function assertArgumentsSupport()
-    {
-        if (! $this->supportsArguments()) {
-            throw new ProgrammingError(
-                'Objects of type "%s" have no arguments',
-                $this->getType()
-            );
-        }
-
-        return $this;
-    }
-
     protected function assertImportsSupport()
     {
         if (! $this->supportsImports()) {
@@ -1208,6 +1242,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 } else {
                     $this->vars = CustomVariables::loadForStoredObject($this);
                 }
+
+                if ($this->getShortTableName() === 'host') {
+                    $this->vars->setOverrideKeyName(
+                        $this->getConnection()->settings()->override_services_varname
+                    );
+                }
             } else {
                 $this->vars = new CustomVariables();
             }
@@ -1284,6 +1324,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     protected function beforeStore()
     {
         $this->resolveUnresolvedRelatedProperties();
+        if ($this->gotImports()) {
+            $this->imports()->getObjects();
+        }
     }
 
     public function onInsert()
@@ -1298,6 +1341,11 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         $this->storeRelatedObjects();
     }
 
+    public function onStore()
+    {
+        $this->notifyResolvers();
+    }
+
     /**
      * @return self
      */
@@ -1351,13 +1399,17 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      */
     protected function storeArguments()
     {
-        if ($this->supportsArguments()) {
-            $this->arguments !== null && $this->arguments()->store();
+        if ($this instanceof ObjectWithArguments) {
+            $this->gotArguments() && $this->arguments()->store();
         }
 
         return $this;
     }
 
+    protected function notifyResolvers()
+    {
+    }
+
     /**
      * @return self
      */
@@ -1548,6 +1600,21 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     protected function renderImports()
     {
+        if (! $this->supportsImports()) {
+            return '';
+        }
+
+        $ret = '';
+        foreach ($this->getImports() as $name) {
+            $ret .= '    import ' . c::renderString($name) . "\n";
+        }
+
+        if ($ret !== '') {
+            $ret .= "\n";
+        }
+        return $ret;
+
+
         // TODO: parent_host ORDERed by weigth...
         if ($this->supportsImports()) {
             return $this->imports()->toConfigString();
@@ -1925,11 +1992,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      */
     protected function renderArguments()
     {
-        if ($this->supportsArguments()) {
-            return $this->arguments()->toConfigString();
-        } else {
-            return '';
-        }
+        return '';
     }
 
     protected function renderRelatedSets()
@@ -2214,7 +2277,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
     }
 
-    protected static function classByType($type)
+    public static function classByType($type)
     {
         // allow for icinga_host and host
         $type = lcfirst(preg_replace('/^icinga_/', '', $type));
@@ -2238,6 +2301,10 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             $type = 'serviceSet';
         } elseif ($type === 'apiuser') {
             $type = 'apiUser';
+        } elseif ($type === 'host_template_choice') {
+            $type = 'templateChoiceHost';
+        } elseif ($type === 'service_template_choice') {
+            $type = 'TemplateChoiceService';
         }
 
         return 'Icinga\\Module\\Director\\Objects\\' . $prefix . ucfirst($type);
@@ -2306,6 +2373,33 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
     }
 
+    /**
+     * @param $type
+     * @param Db $db
+     * @return IcingaObject[]
+     * @throws ProgrammingError
+     */
+    public static function loadAllExternalObjectsByType($type, Db $db)
+    {
+        /** @var IcingaObject $class */
+        $class = self::classByType($type);
+        $dummy = $class::create();
+
+        if (is_array($dummy->getKeyName())) {
+            throw new ProgrammingError(
+                'There is no support for loading external objects of type "%s"',
+                $type
+            );
+        } else {
+            $query = $db->getDbAdapter()
+                ->select()
+                ->from($dummy->getTableName())
+                ->where('object_type = ?', 'external_object');
+
+            return $class::loadAll($db, $query, 'object_name');
+        }
+    }
+
     public static function fromJson($json, Db $connection = null)
     {
         return static::fromPlainObject(json_decode($json), $connection);
@@ -2335,22 +2429,59 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     }
 
     // TODO: with rules? What if I want to override vars? Drop in favour of vars.x?
-    public function merge(IcingaObject $object)
+    public function merge(IcingaObject $object, $replaceVars = false)
     {
         $object = clone($object);
 
         if ($object->supportsCustomVars()) {
             $vars = $object->getVars();
-            $object->vars = array();
+            $object->set('vars', []);
         }
 
-        $this->setProperties((array) $object->toPlainObject(null, true));
+        if ($object->supportsGroups()) {
+            $groups = $object->getGroups();
+            $object->set('groups', []);
+        }
+
+        if ($object->supportsImports()) {
+            $imports = $object->listImportNames();
+            $object->set('imports', []);
+        }
+
+        $plain = (array) $object->toPlainObject(false, false);
+        unset($plain['vars']);
+        unset($plain['groups']);
+        unset($plain['imports']);
+        foreach ($plain as $p => $v) {
+            if ($v === null) {
+                // We want default values, but no null values
+                continue;
+            }
+
+            $this->set($p, $v);
+        }
 
         if ($object->supportsCustomVars()) {
             $myVars = $this->vars();
-            /** @var CustomVariables $vars */
-            foreach ($vars as $key => $var) {
-                $myVars->set($key, $var);
+            if ($replaceVars) {
+                $this->set('vars', $vars);
+            } else {
+                /** @var CustomVariables $vars */
+                foreach ($vars as $key => $var) {
+                    $myVars->set($key, $var);
+                }
+            }
+        }
+
+        if ($object->supportsGroups()) {
+            if (! empty($groups)) {
+                $this->set('groups', $groups);
+            }
+        }
+
+        if ($object->supportsImports()) {
+            if (! empty($imports)) {
+                $this->set('imports', $imports);
             }
         }
 
@@ -2401,12 +2532,6 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 }
             }
 
-            if ($chosenProperties !== null) {
-                if (! in_array($v, $chosenProperties)) {
-                    continue;
-                }
-            }
-
             // TODO: Do not ship null properties based on flag?
             if (!$skipDefaults || $this->differsFromDefaultValue($k, $v)) {
                 if ($k === 'disabled' || $this->propertyIsBoolean($k)) {
@@ -2434,8 +2559,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
-        if ($this->supportsArguments()) {
-            // TODO: resolve
+        if ($this instanceof ObjectWithArguments) {
             $props['arguments'] = $this->arguments()->toPlainObject(
                 $resolved,
                 $skipDefaults
@@ -2452,9 +2576,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
         if ($this->supportsImports()) {
             if ($resolved) {
-                $props['imports'] = array();
+                $props['imports'] = [];
             } else {
-                $props['imports'] = $this->imports()->listImportNames();
+                $props['imports'] = $this->listImportNames();
             }
         }
 
@@ -2508,11 +2632,50 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
+        if ($chosenProperties !== null) {
+            $chosen = [];
+            foreach ($chosenProperties as $k) {
+                if (array_key_exists($k, $props)) {
+                    $chosen[$k] = $props[$k];
+                }
+            }
+
+            $props = $chosen;
+        }
         ksort($props);
 
         return (object) $props;
     }
 
+    public function listImportNames()
+    {
+        if ($this->gotImports()) {
+            return $this->imports()->listImportNames();
+        } else {
+            return $this->templateTree()->listParentNamesFor($this);
+        }
+    }
+
+    public function listImportIds()
+    {
+        return $this->templateTree()->listParentIdsFor($this);
+    }
+
+    public function listAncestorIds()
+    {
+        return $this->templateTree()->listAncestorIdsFor($this);
+    }
+
+    protected function templateTree()
+    {
+        return $this->templates()->tree();
+    }
+
+    protected function templates()
+    {
+        return IcingaTemplateRepository::instanceByObject($this, $this->getConnection());
+    }
+
     protected function differsFromDefaultValue($key, $value)
     {
         if (array_key_exists($key, $this->defaultProperties)) {
@@ -2549,7 +2712,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     public function getOnDeleteUrl()
     {
-        return 'director/' . strtolower($this->getShortTableName()) . 's';
+        $plural= preg_replace('/cys$/', 'cies', strtolower($this->getShortTableName()) . 's');
+        return 'director/' . $plural;
     }
 
     public function toJson(
@@ -2607,7 +2771,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
-        if ($this->supportsArguments()) {
+        if ($this instanceof ObjectWithArguments) {
             $args = $this->arguments()->toUnmodifiedPlainObject();
             if (! empty($args)) {
                 $props['arguments'] = $args;
@@ -2666,7 +2830,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         unset($this->groups);
         unset($this->imports);
         unset($this->ranges);
-        unset($this->arguments);
+        if ($this instanceof ObjectWithArguments) {
+            $this->unsetArguments();
+        }
 
         parent::__destruct();
     }
