@@ -27,6 +27,7 @@ class IcingaService extends IcingaObject
         'check_period_id'       => null,
         'check_interval'        => null,
         'retry_interval'        => null,
+        'check_timeout'         => null,
         'enable_notifications'  => null,
         'enable_active_checks'  => null,
         'enable_passive_checks' => null,
@@ -47,6 +48,7 @@ class IcingaService extends IcingaObject
         'apply_for'             => null,
         'use_var_overrides'     => null,
         'assign_filter'         => null,
+        'template_choice_id'    => null,
     );
 
     protected $relations = array(
@@ -57,6 +59,7 @@ class IcingaService extends IcingaObject
         'check_period'     => 'IcingaTimePeriod',
         'command_endpoint' => 'IcingaEndpoint',
         'zone'             => 'IcingaZone',
+        'template_choice'  => 'IcingaTemplateChoiceService',
     );
 
     protected $booleans = array(
@@ -73,6 +76,7 @@ class IcingaService extends IcingaObject
 
     protected $intervalProperties = array(
         'check_interval' => 'check_interval',
+        'check_timeout'  => 'check_timeout',
         'retry_interval' => 'retry_interval',
     );
 
@@ -88,6 +92,8 @@ class IcingaService extends IcingaObject
 
     protected $supportsSets = true;
 
+    protected $supportsChoices = true;
+
     protected $supportedInLegacy = true;
 
     protected $keyName = array('host_id', 'service_set_id', 'object_name');
@@ -340,13 +346,13 @@ class IcingaService extends IcingaObject
             return $output;
         }
 
-        // In case use_agent isn't defined, do nothing
-        // TODO: what if we inherit use_agent and override it with 'n'?
-        if ($this->use_agent !== 'y') {
+        if ($this->use_agent === 'y') {
+            return $output . c::renderKeyValue('command_endpoint', 'host_name');
+        } elseif ($this->use_agent === 'n') {
+            return $output . c::renderKeyValue('command_endpoint', c::renderPhpValue(null));
+        } else {
             return $output;
         }
-
-        return $output . c::renderKeyValue('command_endpoint', 'host_name');
     }
 
     /**
@@ -367,6 +373,11 @@ class IcingaService extends IcingaObject
         return '';
     }
 
+    protected function renderTemplate_choice_id()
+    {
+        return '';
+    }
+
     protected function renderLegacyDisplay_Name()
     {
         // @codingStandardsIgnoreEnd
@@ -404,6 +415,21 @@ class IcingaService extends IcingaObject
         return $zone;
     }
 
+    public function createWhere()
+    {
+        $where = parent::createWhere();
+        if (! $this->hasBeenLoadedFromDb()) {
+            if (null === $this->get('service_set_id')
+                && null === $this->get('host_id')
+                && null === $this->get('id')
+            ) {
+                $where .= " AND object_type = 'template'";
+            }
+        }
+
+        return $where;
+    }
+
     // TODO: Duplicate code, clean this up, split it into multiple methods
     public static function enumProperties(
         Db $connection = null,
@@ -426,6 +452,9 @@ class IcingaService extends IcingaObject
             }
 
             if (substr($prop, -3) === '_id') {
+                if ($prop === 'template_choice_id') {
+                    continue;
+                }
                 $prop = substr($prop, 0, -3);
             }
 
