@@ -3,23 +3,28 @@
 namespace Icinga\Module\Director\Objects;
 
 use Icinga\Module\Director\Data\Db\DbObjectWithSettings;
+use Icinga\Module\Director\Hook\PropertyModifierHook;
+use Icinga\Module\Director\Objects\Extension\PriorityColumn;
 
 class ImportRowModifier extends DbObjectWithSettings
 {
+    use PriorityColumn;
+
     protected $table = 'import_row_modifier';
 
     protected $keyName = 'id';
 
     protected $autoincKeyName = 'id';
 
-    protected $defaultProperties = array(
+    protected $defaultProperties = [
         'id'              => null,
         'source_id'       => null,
         'property_name'   => null,
         'provider_class'  => null,
         'target_property' => null,
         'priority'        => null,
-    );
+        'description'     => null,
+    ];
 
     protected $settingsTable = 'import_row_modifier_setting';
 
@@ -30,13 +35,27 @@ class ImportRowModifier extends DbObjectWithSettings
     public function getInstance()
     {
         if ($this->hookInstance === null) {
-            $obj = new $this->provider_class;
+            $class = $this->get('provider_class');
+            /** @var PropertyModifierHook $obj */
+            $obj = new $class;
             $obj->setSettings($this->getSettings());
-            $obj->setTargetProperty($this->target_property);
+            $obj->setTargetProperty($this->get('target_property'));
             $obj->setDb($this->connection);
             $this->hookInstance = $obj;
         }
 
         return $this->hookInstance;
     }
+
+    protected function beforeStore()
+    {
+        if (! $this->hasBeenLoadedFromDb()) {
+            $this->setNextPriority('source_id');
+        }
+    }
+
+    protected function onInsert()
+    {
+        $this->refreshPriortyProperty();
+    }
 }
