@@ -3,16 +3,19 @@
 namespace Icinga\Module\Director\Objects;
 
 use Icinga\Module\Director\Data\Db\DbObject;
+use Icinga\Module\Director\Objects\Extension\PriorityColumn;
 
 class SyncProperty extends DbObject
 {
+    use PriorityColumn;
+
     protected $table = 'sync_property';
 
     protected $keyName = 'id';
 
     protected $autoincKeyName = 'id';
 
-    protected $defaultProperties = array(
+    protected $defaultProperties = [
         'id'                => null,
         'rule_id'           => null,
         'source_id'         => null,
@@ -21,5 +24,17 @@ class SyncProperty extends DbObject
         'priority'          => null,
         'filter_expression' => null,
         'merge_policy'      => null
-    );
+    ];
+
+    protected function beforeStore()
+    {
+        if (! $this->hasBeenLoadedFromDb()) {
+            $this->setNextPriority('rule_id');
+        }
+    }
+
+    protected function onInsert()
+    {
+        $this->refreshPriortyProperty();
+    }
 }
