@@ -27,6 +27,7 @@ class SyncRule extends DbObject
         'sync_state'         => 'unknown',
         'last_error_message' => null,
         'last_attempt'       => null,
+        'description'        => null,
     );
 
     private $sync;
@@ -110,22 +111,6 @@ class SyncRule extends DbObject
         return $db->fetchOne($query);
     }
 
-    public function getPriorityForNextProperty()
-    {
-        if (! $this->hasBeenLoadedFromDb()) {
-            return 1;
-        }
-        
-        $db = $this->getDb();
-        return $db->fetchOne(
-            $db->select()
-                ->from(
-                    array('p' => 'sync_property'),
-                    array('priority' => '(CASE WHEN MAX(p.priority) IS NULL THEN 1 ELSE MAX(p.priority) + 1 END)')
-                )->where('p.rule_id = ?', $this->get('id'))
-        );
-    }
-
     public function matches($row)
     {
         if ($this->get('filter_expression') === null) {
@@ -218,7 +203,10 @@ class SyncRule extends DbObject
         return $this->sync;
     }
 
-    protected function filter()
+    /**
+     * @return Filter
+     */
+    public function filter()
     {
         if ($this->filter === null) {
             $this->filter = Filter::fromQueryString($this->get('filter_expression'));
@@ -345,6 +333,12 @@ class SyncRule extends DbObject
         return $this->hasCombinedKey;
     }
 
+    public function hasSyncProperties()
+    {
+        $properties = $this->getSyncProperties();
+        return ! empty($properties);
+    }
+
     public function getSyncProperties()
     {
         if (! $this->hasBeenLoadedFromDb()) {
@@ -367,7 +361,7 @@ class SyncRule extends DbObject
             $db->select()
                ->from('sync_property')
                ->where('rule_id = ?', $this->get('id'))
-               ->order('priority DESC')
+               ->order('priority ASC')
         );
     }
 }
