@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Web\Form;
 
 use Exception;
 use Icinga\Data\Filter\Filter;
+use Icinga\Data\Filter\FilterChain;
 use Icinga\Data\Filter\FilterExpression;
 use Icinga\Exception\IcingaException;
 use Icinga\Module\Director\Objects\IcingaCommand;
@@ -11,6 +12,7 @@ use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Objects\DirectorDatafield;
 use Icinga\Module\Director\Objects\IcingaService;
+use Icinga\Module\Director\Objects\ObjectApplyMatches;
 use stdClass;
 use Zend_Db_Select as ZfSelect;
 use Zend_Form_Element as ZfElement;
@@ -44,7 +46,7 @@ class IcingaObjectFieldLoader
         $this->db = $this->connection->getDbAdapter();
     }
 
-    public function addFieldsToForm(QuickForm $form)
+    public function addFieldsToForm(DirectorObjectForm $form)
     {
         if ($this->fields || $this->object->supportsFields()) {
             $this->attachFieldsToForm($form);
@@ -166,11 +168,11 @@ class IcingaObjectFieldLoader
     /**
      * Get the form elements for our fields
      *
-     * @param QuickForm $form Optional
+     * @param DirectorObjectForm $form Optional
      *
      * @return ZfElement[]
      */
-    public function getElements(QuickForm $form = null)
+    public function getElements(DirectorObjectForm $form = null)
     {
         if ($this->elements === null) {
             $this->elements = $this->createElements($form);
@@ -183,11 +185,11 @@ class IcingaObjectFieldLoader
     /**
      * Prepare the form elements for our fields
      *
-     * @param QuickForm $form Optional
+     * @param DirectorObjectForm $form Optional
      *
      * @return self
      */
-    public function prepareElements(QuickForm $form = null)
+    public function prepareElements(DirectorObjectForm $form = null)
     {
         if ($this->object->supportsFields()) {
             $this->getElements($form);
@@ -240,14 +242,6 @@ class IcingaObjectFieldLoader
         $kill = array();
         $columns = array();
         $object = $this->object;
-
-        $object->invalidateResolveCache();
-        $vars = $object::fromPlainObject(
-            $object->toPlainObject(true),
-            $object->getConnection()
-        )->vars()->flatten();
-
-        $prefixedVars = (object) array();
         if ($object instanceof IcingaHost) {
             $prefix = 'host.vars.';
         } elseif ($object instanceof IcingaService) {
@@ -256,11 +250,19 @@ class IcingaObjectFieldLoader
             return $elements;
         }
 
+        $object->invalidateResolveCache();
+        $vars = $object::fromPlainObject(
+            $object->toPlainObject(true),
+            $object->getConnection()
+        )->getVars();
+
+        $prefixedVars = (object) array();
         foreach ($vars as $k => $v) {
             $prefixedVars->{$prefix . $k} = $v;
         }
 
         foreach ($filters as $key => $filter) {
+            ObjectApplyMatches::fixFilterColumns($filter);
             /** @var $filter FilterChain|FilterExpression */
             foreach ($filter->listFilteredColumns() as $column) {
                 $column = substr($column, strlen($prefix));
@@ -323,11 +325,11 @@ class IcingaObjectFieldLoader
     /**
      * Get the form elements based on the given form
      *
-     * @param QuickForm $form
+     * @param DirectorObjectForm $form
      *
      * @return ZfElement[]
      */
-    protected function createElements(QuickForm $form)
+    protected function createElements(DirectorObjectForm $form)
     {
         $elements = array();
 
@@ -386,7 +388,7 @@ class IcingaObjectFieldLoader
 
             if ($command) {
                 $cmdLoader = new static($command);
-                $cmdFields = $cmdLoader->getFields($command);
+                $cmdFields = $cmdLoader->getFields();
                 foreach ($cmdFields as $varname => $field) {
                     if (! array_key_exists($varname, $fields)) {
                         $fields[$varname] = $field;
@@ -433,7 +435,7 @@ class IcingaObjectFieldLoader
      */
     protected function getIdsForObjectList($objectList)
     {
-        $ids = array();
+        $ids = [];
         foreach ($objectList as $object) {
             if ($object->hasBeenLoadedFromDb()) {
                 $ids[] = $object->get('id');
@@ -445,14 +447,11 @@ class IcingaObjectFieldLoader
 
     public function fetchFieldDetailsForObject(IcingaObject $object)
     {
-        if (! $object->hasBeenLoadedFromDb()) {
-            //return array();
+        $ids = $object->listAncestorIds();
+        if ($id = $object->getProperty('id')) {
+            $ids[] = $id;
         }
-
-        $pathIds = $object->templateResolver()->listInheritancePathIds();
-        return $this->fetchFieldDetailsForIds(
-            array_unique($pathIds)
-        );
+        return $this->fetchFieldDetailsForIds($ids);
     }
 
     /***
@@ -463,7 +462,7 @@ class IcingaObjectFieldLoader
     protected function fetchFieldDetailsForIds($objectIds)
     {
         if (empty($objectIds)) {
-            return array();
+            return [];
         }
 
         $query = $this->prepareSelectForIds($objectIds);
@@ -507,12 +506,12 @@ class IcingaObjectFieldLoader
     }
 
     /**
-     * Fetches fields for a given List of objects from the database
+     * Fetches fields for a given object
      *
      * Gives a list indexed by object id, with each entry being a list of that
      * objects DirectorDatafield instances indexed by variable name
      *
-     * @param IcingaObject[] $objectList List of objects
+     * @param IcingaObject $object
      *
      * @return array
      */
