@@ -30,11 +30,35 @@
             this.module.on('dblclick', '.director-suggest', this.suggestionDoubleClick);
             this.module.on('focus', '.director-suggest', this.enterSuggestionField);
             this.module.on('focusout', '.director-suggest', this.leaveSuggestionField);
-            this.module.on('click', '.director-suggestions li', this.clickSuggestion);
+            this.module.on('mousedown', '.director-suggestions li', this.clickSuggestion);
+            this.module.on('dblclick', 'ul.tabs a', this.tabWantsFullscreen);
             this.module.on('change', 'form input.autosubmit, form select.autosubmit', this.setAutoSubmitted);
             this.module.icinga.logger.debug('Director module initialized');
         },
 
+        tabWantsFullscreen: function(ev) {
+            var icinga = this.module.icinga;
+            var $a, $container, id;
+
+            if (icinga.ui.isOneColLayout()) {
+                return;
+            }
+
+            $a = $(ev.currentTarget);
+            $container = $a.closest('.container');
+            id = $container.attr('id');
+
+            icinga.loader.stopPendingRequestsFor($container);
+            if (id === 'col2') {
+                icinga.ui.moveToLeft();
+            }
+
+            icinga.ui.layout1col();
+            icinga.history.pushCurrentState();
+            ev.preventDefault();
+            ev.stopPropagation();
+        },
+
         /**
          * Autocomplete/suggestion eventhandler
          *
@@ -62,10 +86,10 @@
                     this.removeSuggestionList($el);
                     $el.trigger('change');
                 }
-            } else if (ev.keyCode == 27) {
+            } else if (ev.keyCode === 27) {
                 // ESC key pressed. Remove suggestions if any
                 this.removeSuggestionList($el);
-            } else if (ev.keyCode == 39) {
+            } else if (ev.keyCode === 39) {
                 /**
                  * RIGHT ARROW key pressed. In case there are any suggestions:
                  * - let's choose the active one (if set)
@@ -79,7 +103,7 @@
                         ev.preventDefault();
                     }
                 }
-            } else if (ev.keyCode == 38 ) {
+            } else if (ev.keyCode === 38 ) {
                 /**
                  * UP ARROW key pressed. In any case:
                  * - stop the event
@@ -88,7 +112,7 @@
                 ev.stopPropagation();
                 ev.preventDefault();
                 this.activatePrevSuggestion($el);
-            } else if (ev.keyCode == 40 ) { // down
+            } else if (ev.keyCode === 40 ) { // down
                 /**
                  * DOWN ARROW key pressed. In any case:
                  * - stop the event
@@ -103,7 +127,7 @@
         suggestionDoubleClick: function (ev)
         {
             var $el = $(ev.currentTarget);
-            this.getSuggestionList($el)
+            this.getSuggestionList($el);
         },
 
         /**
@@ -116,17 +140,17 @@
         autoSuggest: function(ev)
         {
             // Ignore special keys, most of them have already been handled on 'keydown'
-            if (ev.keyCode == 9 || // TAB
-                ev.keyCode == 13 || // RETURN
-                ev.keyCode == 27 || // ESC
-                ev.keyCode == 37 || // LEFT ARROW
-                ev.keyCode == 38 || // UP ARROW
-                ev.keyCode == 39 ) { // RIGHT ARROW
+            if (ev.keyCode === 9 || // TAB
+                ev.keyCode === 13 || // RETURN
+                ev.keyCode === 27 || // ESC
+                ev.keyCode === 37 || // LEFT ARROW
+                ev.keyCode === 38 || // UP ARROW
+                ev.keyCode === 39 ) { // RIGHT ARROW
                 return;
             }
 
             var $el = $(ev.currentTarget);
-            if (ev.keyCode == 40) { // DOWN ARROW
+            if (ev.keyCode === 40) { // DOWN ARROW
                 this.getSuggestionList($el);
             } else {
                 this.getSuggestionList($el, true);
@@ -234,6 +258,7 @@
             } else {
                 $ul = $('<ul class="director-suggestions"></ul>');
                 $ul.insertAfter($input);
+                $ul.css({width: $input.css('width')});
                 return this.refreshSuggestionList($ul, $input);
             }
         },
@@ -250,7 +275,8 @@
         {
             $suggestions.load(this.module.icinga.config.baseUrl + '/director/suggest', {
                 value: $el.val(),
-                context: $el.data('suggestion-context')
+                context: $el.data('suggestion-context'),
+                for_host: $el.data('suggestion-for-host')
             }, function (responseText, textStatus, jqXHR) {
                 var $li = $suggestions.find('li');
                 if ($li.length) {
@@ -330,11 +356,12 @@
         },
 
         /**
-         * Show suggestions when arriving to an empte autocompletion field
+         * Show suggestions when arriving to an empty auto-completion field
          *
          * @param ev
          */
         enterSuggestionField: function(ev) {
+            return;
             var $el = $(ev.currentTarget);
             if ($el.val() === '' || $el.val().match(/\.$/)) {
                 this.getSuggestionList($el)
@@ -347,7 +374,7 @@
          * @param ev
          */
         leaveSuggestionField: function(ev) {
-            return;
+//            return;
             var _this = this;
             setTimeout(function() {
                 _this.removeSuggestionList($(ev.currentTarget));
@@ -478,16 +505,24 @@
                 ev.preventDefault();
                 ev.stopPropagation();
                 return false;
-            } else if (el.name.match(/__MOVE_REMOVE$/)) {
-                // TODO: skipping for now, wasn't able to prevent web2 form
-                //       submission once removed
-                /*
-                var $li = $(el).closest('li').remove();
+            } else if (el.name.match(/__REMOVE$/)) {
+                $li = $(el).closest('li');
+                if ($li.find('.autosubmit').length) {
+                    // Autosubmit element, let the server handle this
+                    return true;
+                }
+
+                $li.remove();
                 this.fixRelatedActions($li.closest('ul'));
                 ev.preventDefault();
                 ev.stopPropagation();
                 return false;
-                */
+            } else if (el.name.match(/__DROP_DOWN$/)) {
+                ev.preventDefault();
+                ev.stopPropagation();
+                var $el = $(ev.currentTarget).closest('li').find('input[type=text]');
+                this.getSuggestionList($el);
+                return false;
             }
         },
 
@@ -524,6 +559,14 @@
             if ($input.closest('form.editor').length) {
                return;
             }
+            var $set = $input.closest('.extensible-set');
+            if ($set.length) {
+                var $textInputs = $('input[type=text]', $set);
+                if ($textInputs.length > 1) {
+                    $textInputs.not(':first').attr('tabIndex', '-1');
+                }
+            }
+
             var $dd = $input.closest('dd');
             $dd.find('p.description').show();
             if ($dd.attr('id') && $dd.attr('id').match(/button/)) {
@@ -533,23 +576,19 @@
             var $dt = $dd.prev();
             var $form = $dd.closest('form');
 
+            var $desc = $dd.find('p.description');
+            if ($desc.length) {
+                $form.css({ marginBottom: ($desc.height() + 48) + 'px' });
+            }
+
             $form.find('dt, dd, li').removeClass('active');
             $li.addClass('active');
             $dt.addClass('active');
             $dd.addClass('active');
-            $dd.find('p.description.fading-out')
-                .stop(true)
-                .removeClass('fading-out')
-                .fadeIn('fast');
 
             $form.find('dd').not($dd)
                 .find('p.description')
-                .not('.fading-out')
-                .addClass('fading-out')
-                .delay(2000)
-                .fadeOut('slow', function() {
-                    $(this).removeClass('fading-out').hide()
-                });
+                .hide();
         },
 
         highlightFormErrors: function($container)
@@ -626,12 +665,12 @@
             this.highlightFormErrors($container);
             this.scrollHighlightIntoView($container);
             this.scrollActiveRowIntoView($container);
+            this.highlightActiveDashlet($container);
             this.hideInactiveFormDescriptions($container);
             if (iid = $container.data('activeExtensibleEntry')) {
                 $('#' + iid).focus();
                 $container.removeData('activeExtensibleEntry');
             }
-
             // Disabled for now
             // this.alignDetailLinks();
             if (! this.containerIsAutorefreshed($container) && ! this.containerIsAutoSubmitted($container)) {
@@ -639,6 +678,32 @@
             }
         },
 
+        highlightActiveDashlet: function($container)
+        {
+            if (this.module.icinga.ui.isOneColLayout()) {
+                return;
+            }
+
+            var url, $actions, $match;
+            var id = $container.attr('id');
+            if (id === 'col1') {
+                url = $('#col2').data('icingaUrl');
+                $actions = $('.main-actions', $container);
+            } else if (id === 'col2') {
+                url = $container.data('icingaUrl');
+                $actions = $('.main-actions', $('#col1'));
+            }
+            if (! $actions.length) {
+                return;
+            }
+
+            $match = $('li a[href*="' + url + '"]', $actions);
+            if ($match.length) {
+                $('li a.active', $actions).removeClass('active');
+                $match.first().addClass('active');
+            }
+        },
+
         restoreContainerFieldsets: function($container)
         {
             var self = this;
@@ -729,4 +794,3 @@
     Icinga.availableModules.director = Director;
 
 }(Icinga));
-
