@@ -1,9 +1,16 @@
 <?php
 
-use Icinga\Application\Icinga;
+use Icinga\Application\Modules\Module;
+use dipl\Loader\CompatLoader;
+
+if (version_compare(PHP_VERSION, '5.4.0') < 0) {
+    include __DIR__ . '/run-php5.3.php';
+    return;
+}
 
 $prefix = '\\Icinga\\Module\\Director\\';
 
+/** @var Module $this */
 $this->provideHook('monitoring/HostActions');
 $this->provideHook('monitoring/ServiceActions');
 
@@ -38,6 +45,9 @@ $this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\Pro
 $this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierMakeBoolean');
 $this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierJsonDecode');
 $this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierToInt');
+$this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierLConfCustomVar');
+$this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierArrayFilter');
+$this->provideHook('director/PropertyModifier', $prefix . 'PropertyModifier\\PropertyModifierCombine');
 
 $this->provideHook('director/Job', $prefix . 'Job\\HousekeepingJob');
 $this->provideHook('director/Job', $prefix . 'Job\\ConfigJob');
@@ -45,3 +55,6 @@ $this->provideHook('director/Job', $prefix . 'Job\\ImportJob');
 $this->provideHook('director/Job', $prefix . 'Job\\SyncJob');
 
 $this->provideHook('cube/Actions', 'CubeLinks');
+
+require_once __DIR__ . '/library/vendor/ipl/Loader/CompatLoader.php';
+CompatLoader::delegateLoadingToIcingaWeb($this->app);
