@@ -110,6 +110,7 @@ CREATE TABLE director_datalist_entry (
   entry_name VARCHAR(255) COLLATE utf8_bin NOT NULL,
   entry_value TEXT DEFAULT NULL,
   format enum ('string', 'expression', 'json'),
+  allowed_roles VARCHAR(255) DEFAULT NULL,
   PRIMARY KEY (list_id, entry_name),
   CONSTRAINT director_datalist_value_datalist
     FOREIGN KEY datalist (list_id)
@@ -421,6 +422,18 @@ CREATE TABLE icinga_endpoint_inheritance (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+CREATE TABLE icinga_host_template_choice (
+  id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
+  object_name VARCHAR(64) NOT NULL,
+  description TEXT DEFAULT NULL,
+  min_required SMALLINT UNSIGNED NOT NULL DEFAULT 0,
+  max_allowed SMALLINT UNSIGNED NOT NULL DEFAULT 1,
+  required_template_id INT(10) UNSIGNED DEFAULT NULL,
+  allowed_roles VARCHAR(255) DEFAULT NULL,
+  PRIMARY KEY (id),
+  UNIQUE KEY (object_name)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
 CREATE TABLE icinga_host (
   id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
   object_name VARCHAR(255) NOT NULL,
@@ -434,6 +447,7 @@ CREATE TABLE icinga_host (
   check_period_id INT(10) UNSIGNED DEFAULT NULL,
   check_interval VARCHAR(8) DEFAULT NULL,
   retry_interval VARCHAR(8) DEFAULT NULL,
+  check_timeout SMALLINT UNSIGNED DEFAULT NULL,
   enable_notifications ENUM('y', 'n') DEFAULT NULL,
   enable_active_checks ENUM('y', 'n') DEFAULT NULL,
   enable_passive_checks ENUM('y', 'n') DEFAULT NULL,
@@ -454,6 +468,7 @@ CREATE TABLE icinga_host (
   master_should_connect ENUM('y', 'n') DEFAULT NULL,
   accept_config ENUM('y', 'n') DEFAULT NULL,
   api_key VARCHAR(40) DEFAULT NULL,
+  template_choice_id INT(10) UNSIGNED DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE INDEX object_name (object_name),
   UNIQUE INDEX api_key (api_key),
@@ -482,6 +497,11 @@ CREATE TABLE icinga_host (
     FOREIGN KEY command_endpoint (command_endpoint_id)
     REFERENCES icinga_endpoint (id)
     ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_host_template_choice
+    FOREIGN KEY choice (template_choice_id)
+    REFERENCES icinga_host_template_choice (id)
+    ON DELETE SET NULL
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -537,6 +557,13 @@ CREATE TABLE icinga_host_var (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+ALTER TABLE icinga_host_template_choice
+  ADD CONSTRAINT host_template_choice_required_template
+    FOREIGN KEY required_template (required_template_id)
+    REFERENCES icinga_host (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE;
+
 CREATE TABLE icinga_service_set (
   id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
   object_name VARCHAR(128) NOT NULL,
@@ -553,6 +580,18 @@ CREATE TABLE icinga_service_set (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+CREATE TABLE icinga_service_template_choice (
+  id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
+  object_name VARCHAR(64) NOT NULL,
+  description TEXT DEFAULT NULL,
+  min_required SMALLINT UNSIGNED NOT NULL DEFAULT 0,
+  max_allowed SMALLINT UNSIGNED NOT NULL DEFAULT 1,
+  required_template_id INT(10) UNSIGNED DEFAULT NULL,
+  allowed_roles VARCHAR(255) DEFAULT NULL,
+  PRIMARY KEY (id),
+  UNIQUE KEY (object_name)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
 CREATE TABLE icinga_service (
   id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
   object_name VARCHAR(255) NOT NULL,
@@ -566,6 +605,7 @@ CREATE TABLE icinga_service (
   check_period_id INT(10) UNSIGNED DEFAULT NULL,
   check_interval VARCHAR(8) DEFAULT NULL,
   retry_interval VARCHAR(8) DEFAULT NULL,
+  check_timeout SMALLINT UNSIGNED DEFAULT NULL,
   enable_notifications ENUM('y', 'n') DEFAULT NULL,
   enable_active_checks ENUM('y', 'n') DEFAULT NULL,
   enable_passive_checks ENUM('y', 'n') DEFAULT NULL,
@@ -586,6 +626,7 @@ CREATE TABLE icinga_service (
   apply_for VARCHAR(255) DEFAULT NULL,
   use_var_overrides ENUM('y', 'n') DEFAULT NULL,
   assign_filter TEXT DEFAULT NULL,
+  template_choice_id INT(10) UNSIGNED DEFAULT NULL,
   PRIMARY KEY (id),
   UNIQUE KEY object_key (object_name, host_id),
   CONSTRAINT icinga_service_host
@@ -622,6 +663,11 @@ CREATE TABLE icinga_service (
     FOREIGN KEY service_set (service_set_id)
     REFERENCES icinga_service_set (id)
     ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_service_template_choice
+    FOREIGN KEY choice (template_choice_id)
+    REFERENCES icinga_service_template_choice (id)
+    ON DELETE SET NULL
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -677,6 +723,13 @@ CREATE TABLE icinga_service_field (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+ALTER TABLE icinga_service_template_choice
+  ADD CONSTRAINT service_template_choice_required_template
+    FOREIGN KEY required_template (required_template_id)
+    REFERENCES icinga_service (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE;
+
 CREATE TABLE icinga_host_service (
   host_id INT(10) UNSIGNED NOT NULL,
   service_id INT(10) UNSIGNED NOT NULL,
@@ -730,7 +783,7 @@ CREATE TABLE icinga_service_set_var (
 CREATE TABLE icinga_hostgroup (
   id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
   object_name VARCHAR(255) NOT NULL,
-  object_type ENUM('object', 'template') NOT NULL,
+  object_type ENUM('object', 'template', 'external_object') NOT NULL,
   disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
   display_name VARCHAR(255) DEFAULT NULL,
   assign_filter TEXT DEFAULT NULL,
@@ -820,6 +873,22 @@ CREATE TABLE icinga_hostgroup_host (
     ON UPDATE CASCADE
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+CREATE TABLE icinga_hostgroup_host_resolved (
+  hostgroup_id INT(10) UNSIGNED NOT NULL,
+  host_id INT(10) UNSIGNED NOT NULL,
+  PRIMARY KEY (hostgroup_id, host_id),
+  CONSTRAINT icinga_hostgroup_host_resolved_host
+  FOREIGN KEY host (host_id)
+  REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_hostgroup_host_resolved_hostgroup
+  FOREIGN KEY hostgroup (hostgroup_id)
+  REFERENCES icinga_hostgroup (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
 CREATE TABLE icinga_hostgroup_parent (
   hostgroup_id INT(10) UNSIGNED NOT NULL,
   parent_hostgroup_id INT(10) UNSIGNED NOT NULL,
@@ -1195,6 +1264,7 @@ CREATE TABLE import_source (
   ) NOT NULL DEFAULT 'unknown',
   last_error_message TEXT DEFAULT NULL,
   last_attempt DATETIME DEFAULT NULL,
+  description TEXT DEFAULT NULL,
   PRIMARY KEY (id),
   INDEX search_idx (key_column)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
@@ -1218,6 +1288,7 @@ CREATE TABLE import_row_modifier (
   target_property VARCHAR(255) DEFAULT NULL,
   provider_class VARCHAR(72) NOT NULL,
   priority SMALLINT UNSIGNED NOT NULL,
+  description TEXT DEFAULT NULL,
   PRIMARY KEY (id),
   KEY search_idx (property_name),
   CONSTRAINT row_modifier_import_source
@@ -1339,6 +1410,7 @@ CREATE TABLE sync_rule (
   ) NOT NULL DEFAULT 'unknown',
   last_error_message TEXT DEFAULT NULL,
   last_attempt DATETIME DEFAULT NULL,
+  description TEXT DEFAULT NULL,
   PRIMARY KEY (id)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -1506,6 +1578,94 @@ CREATE TABLE icinga_user_resolved_var (
     ON UPDATE RESTRICT
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
+CREATE TABLE icinga_dependency (
+  id INT(10) UNSIGNED AUTO_INCREMENT NOT NULL,
+  object_name VARCHAR(255) DEFAULT NULL,
+  object_type ENUM('object', 'template', 'apply') NOT NULL,
+  disabled ENUM('y', 'n') NOT NULL DEFAULT 'n',
+  apply_to ENUM('host', 'service') DEFAULT NULL,
+  parent_host_id INT(10) UNSIGNED DEFAULT NULL,
+  parent_service_id INT(10) UNSIGNED DEFAULT NULL,
+  child_host_id INT(10) UNSIGNED DEFAULT NULL,
+  child_service_id INT(10) UNSIGNED DEFAULT NULL,
+  disable_checks ENUM('y', 'n') DEFAULT NULL,
+  disable_notifications ENUM('y', 'n') DEFAULT NULL,
+  ignore_soft_states ENUM('y', 'n') DEFAULT NULL,
+  period_id INT(10) UNSIGNED DEFAULT NULL,
+  zone_id INT(10) UNSIGNED DEFAULT NULL,
+  assign_filter TEXT DEFAULT NULL,
+  parent_service_by_name VARCHAR(255) DEFAULT NULL,
+  PRIMARY KEY (id),
+  CONSTRAINT icinga_dependency_parent_host
+    FOREIGN KEY parent_host (parent_host_id)
+    REFERENCES icinga_host (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_parent_service
+    FOREIGN KEY parent_service (parent_service_id)
+    REFERENCES icinga_service (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_child_host
+    FOREIGN KEY child_host (child_host_id)
+    REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_child_service
+    FOREIGN KEY child_service (child_service_id)
+    REFERENCES icinga_service (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_period
+    FOREIGN KEY period (period_id)
+    REFERENCES icinga_timeperiod (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_zone
+    FOREIGN KEY zone (zone_id)
+    REFERENCES icinga_zone (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
+CREATE TABLE icinga_dependency_inheritance (
+  dependency_id INT(10) UNSIGNED NOT NULL,
+  parent_dependency_id INT(10) UNSIGNED NOT NULL,
+  weight MEDIUMINT UNSIGNED DEFAULT NULL,
+  PRIMARY KEY (dependency_id, parent_dependency_id),
+  UNIQUE KEY unique_order (dependency_id, weight),
+  CONSTRAINT icinga_dependency_inheritance_dependency
+    FOREIGN KEY dependency (dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_inheritance_parent_dependency
+    FOREIGN KEY parent_dependency (parent_dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8;
+
+CREATE TABLE icinga_dependency_states_set (
+  dependency_id INT(10) UNSIGNED NOT NULL,
+  property ENUM(
+    'OK',
+    'Warning',
+    'Critical',
+    'Unknown',
+    'Up',
+    'Down'
+  ) NOT NULL,
+  merge_behaviour ENUM('override', 'extend', 'blacklist') NOT NULL DEFAULT 'override'
+    COMMENT 'override: = [], extend: += [], blacklist: -= []',
+  PRIMARY KEY (dependency_id, property, merge_behaviour),
+  CONSTRAINT icinga_dependency_states_set_dependency
+    FOREIGN KEY icinga_dependency (dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+) ENGINE=InnoDB;
+
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (129, NOW());
+  VALUES (145, NOW());
