@@ -170,6 +170,7 @@ CREATE TABLE director_datalist_entry (
   entry_name character varying(255) NOT NULL,
   entry_value text DEFAULT NULL,
   format enum_property_format,
+  allowed_roles character varying(255) DEFAULT NULL,
   PRIMARY KEY (list_id, entry_name),
   CONSTRAINT director_datalist_entry_datalist
   FOREIGN KEY (list_id)
@@ -544,6 +545,20 @@ CREATE INDEX endpoint_inheritance_endpoint ON icinga_endpoint_inheritance (endpo
 CREATE INDEX endpoint_inheritance_endpoint_parent ON icinga_endpoint_inheritance (parent_endpoint_id);
 
 
+CREATE TABLE icinga_host_template_choice (
+  id serial,
+  object_name character varying(64) NOT NULL,
+  description text DEFAULT NULL,
+  min_required smallint NOT NULL DEFAULT 0,
+  max_allowed smallint NOT NULL DEFAULT 1,
+  required_template_id integer DEFAULT NULL,
+  allowed_roles character varying(255) DEFAULT NULL,
+  PRIMARY KEY (id)
+);
+
+CREATE UNIQUE INDEX host_template_choice_object_name ON icinga_host_template_choice (object_name);
+CREATE INDEX host_template_choice_required_template ON icinga_host_template_choice (required_template_id);
+
 CREATE TABLE icinga_host (
   id serial,
   object_name character varying(255) NOT NULL,
@@ -557,6 +572,7 @@ CREATE TABLE icinga_host (
   check_period_id integer DEFAULT NULL,
   check_interval character varying(8) DEFAULT NULL,
   retry_interval character varying(8) DEFAULT NULL,
+  check_timeout smallint DEFAULT NULL,
   enable_notifications enum_boolean DEFAULT NULL,
   enable_active_checks enum_boolean DEFAULT NULL,
   enable_passive_checks enum_boolean DEFAULT NULL,
@@ -577,6 +593,7 @@ CREATE TABLE icinga_host (
   master_should_connect enum_boolean DEFAULT NULL,
   accept_config enum_boolean DEFAULT NULL,
   api_key character varying(40) DEFAULT NULL,
+  template_choice_id int DEFAULT NULL,
   PRIMARY KEY (id),
   CONSTRAINT icinga_host_zone
   FOREIGN KEY (zone_id)
@@ -602,9 +619,15 @@ CREATE TABLE icinga_host (
   FOREIGN KEY (command_endpoint_id)
     REFERENCES icinga_endpoint (id)
     ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_host_template_choice
+    FOREIGN KEY (template_choice_id)
+    REFERENCES icinga_host_template_choice (id)
+    ON DELETE SET NULL
     ON UPDATE CASCADE
 );
 
+
 CREATE UNIQUE INDEX object_name_host ON icinga_host (object_name, zone_id);
 CREATE UNIQUE INDEX host_api_key ON icinga_host (api_key);
 CREATE INDEX host_zone ON icinga_host (zone_id);
@@ -612,6 +635,7 @@ CREATE INDEX host_timeperiod ON icinga_host (check_period_id);
 CREATE INDEX host_check_command ON icinga_host (check_command_id);
 CREATE INDEX host_event_command ON icinga_host (event_command_id);
 CREATE INDEX host_command_endpoint ON icinga_host (command_endpoint_id);
+CREATE INDEX host_template_choice ON icinga_host (template_choice_id);
 
 
 CREATE TABLE icinga_host_inheritance (
@@ -679,6 +703,14 @@ CREATE INDEX host_var_host ON icinga_host_var (host_id);
 CREATE INDEX host_var_checksum ON icinga_host_var (checksum);
 
 
+ALTER TABLE icinga_host_template_choice
+  ADD CONSTRAINT host_template_choice_required_template
+    FOREIGN KEY (required_template_id)
+    REFERENCES icinga_host (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE;
+
+
 CREATE TABLE icinga_service_set (
   id serial,
   host_id integer DEFAULT NULL,
@@ -698,6 +730,21 @@ CREATE UNIQUE INDEX service_set_name ON icinga_service_set (object_name, host_id
 CREATE INDEX service_set_host ON icinga_service_set (host_id);
 
 
+CREATE TABLE icinga_service_template_choice (
+  id serial,
+  object_name character varying(64) NOT NULL,
+  description text DEFAULT NULL,
+  min_required smallint NOT NULL DEFAULT 0,
+  max_allowed smallint NOT NULL DEFAULT 1,
+  required_template_id integer DEFAULT NULL,
+  allowed_roles character varying(255) DEFAULT NULL,
+  PRIMARY KEY (id)
+);
+
+CREATE UNIQUE INDEX service_template_choice_object_name ON icinga_service_template_choice (object_name);
+CREATE INDEX service_template_choice_required_template ON icinga_service_template_choice (required_template_id);
+
+
 CREATE TABLE icinga_service (
   id serial,
   object_name character varying(255) NOT NULL,
@@ -711,6 +758,7 @@ CREATE TABLE icinga_service (
   check_period_id integer DEFAULT NULL,
   check_interval character varying(8) DEFAULT NULL,
   retry_interval character varying(8) DEFAULT NULL,
+  check_timeout smallint DEFAULT NULL,
   enable_notifications enum_boolean DEFAULT NULL,
   enable_active_checks enum_boolean DEFAULT NULL,
   enable_passive_checks enum_boolean DEFAULT NULL,
@@ -731,6 +779,7 @@ CREATE TABLE icinga_service (
   apply_for character varying(255) DEFAULT NULL,
   use_var_overrides enum_boolean DEFAULT NULL,
   assign_filter text DEFAULT NULL,
+  template_choice_id int DEFAULT NULL,
   PRIMARY KEY (id),
 -- UNIQUE INDEX object_name (object_name, zone_id),
   CONSTRAINT icinga_service_host
@@ -767,6 +816,11 @@ CREATE TABLE icinga_service (
     FOREIGN KEY (service_set_id)
     REFERENCES icinga_service_set (id)
     ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_service_template_choice
+    FOREIGN KEY (template_choice_id)
+    REFERENCES icinga_service_template_choice (id)
+    ON DELETE SET NULL
     ON UPDATE CASCADE
 );
 
@@ -775,6 +829,7 @@ CREATE INDEX service_timeperiod ON icinga_service (check_period_id);
 CREATE INDEX service_check_command ON icinga_service (check_command_id);
 CREATE INDEX service_event_command ON icinga_service (event_command_id);
 CREATE INDEX service_command_endpoint ON icinga_service (command_endpoint_id);
+CREATE INDEX service_template_choice ON icinga_service (template_choice_id);
 
 
 CREATE TABLE icinga_service_inheritance (
@@ -842,6 +897,14 @@ CREATE INDEX service_field_datafield ON icinga_service_field (datafield_id);
 COMMENT ON COLUMN icinga_service_field.service_id IS 'Makes only sense for templates';
 
 
+ALTER TABLE icinga_service_template_choice
+  ADD CONSTRAINT service_template_choice_required_template
+    FOREIGN KEY (required_template_id)
+    REFERENCES icinga_service (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE;
+
+
 CREATE TABLE icinga_host_service (
   host_id integer NOT NULL,
   service_id integer NOT NULL,
@@ -1016,6 +1079,26 @@ CREATE INDEX hostgroup_host_host ON icinga_hostgroup_host (host_id);
 CREATE INDEX hostgroup_host_hostgroup ON icinga_hostgroup_host (hostgroup_id);
 
 
+CREATE TABLE icinga_hostgroup_host_resolved (
+  hostgroup_id integer NOT NULL,
+  host_id integer NOT NULL,
+  PRIMARY KEY (hostgroup_id, host_id),
+  CONSTRAINT icinga_hostgroup_host_resolved_host
+  FOREIGN KEY (host_id)
+    REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_hostgroup_host_resolved_hostgroup
+  FOREIGN KEY (hostgroup_id)
+    REFERENCES icinga_hostgroup (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE INDEX hostgroup_host_resolved_host ON icinga_hostgroup_host_resolved (host_id);
+CREATE INDEX hostgroup_host_resolved_hostgroup ON icinga_hostgroup_host_resolved (hostgroup_id);
+
+
 CREATE TABLE icinga_hostgroup_parent (
   hostgroup_id integer NOT NULL,
   parent_hostgroup_id integer NOT NULL,
@@ -1316,6 +1399,7 @@ CREATE TABLE import_source (
   import_state enum_sync_state NOT NULL DEFAULT 'unknown',
   last_error_message text NULL DEFAULT NULL,
   last_attempt timestamp with time zone NULL DEFAULT NULL,
+  description text DEFAULT NULL,
   PRIMARY KEY (id)
 );
 
@@ -1344,6 +1428,7 @@ CREATE TABLE import_row_modifier (
   target_property character varying(255) DEFAULT NULL,
   provider_class character varying(72) NOT NULL,
   priority integer NOT NULL,
+  description text DEFAULT NULL,
   PRIMARY KEY (id),
   CONSTRAINT row_modifier_import_source
     FOREIGN KEY (source_id)
@@ -1353,6 +1438,8 @@ CREATE TABLE import_row_modifier (
 );
 
 CREATE INDEX import_row_modifier_search_idx ON import_row_modifier (property_name);
+CREATE UNIQUE INDEX import_row_modifier_prio
+  ON import_row_modifier (source_id, priority);
 
 
 CREATE TABLE import_row_modifier_setting (
@@ -1385,8 +1472,8 @@ CREATE TABLE import_run (
   CONSTRAINT import_run_source
   FOREIGN KEY (source_id)
   REFERENCES import_source (id)
-  ON DELETE RESTRICT
-  ON UPDATE CASCADE,
+  ON DELETE CASCADE
+  ON UPDATE RESTRICT,
   CONSTRAINT import_run_rowset
   FOREIGN KEY (rowset_checksum)
   REFERENCES imported_rowset (checksum)
@@ -1468,6 +1555,7 @@ CREATE TABLE sync_rule (
   sync_state enum_sync_state NOT NULL DEFAULT 'unknown',
   last_error_message text NULL DEFAULT NULL,
   last_attempt timestamp with time zone NULL DEFAULT NULL,
+  description text DEFAULT NULL,
   PRIMARY KEY (id)
 );
 
@@ -1773,6 +1861,102 @@ CREATE INDEX user_resolved_var_user_id ON icinga_user_resolved_var (user_id);
 CREATE INDEX user_resolved_var_schecksum ON icinga_user_resolved_var (checksum);
 
 
+CREATE TABLE icinga_dependency (
+  id serial,
+  object_name character varying(255) NOT NULL,
+  object_type enum_object_type_all NOT NULL,
+  disabled enum_boolean DEFAULT 'n',
+  apply_to enum_host_service NULL DEFAULT NULL,
+  parent_host_id integer DEFAULT NULL,
+  parent_service_id integer DEFAULT NULL,
+  child_host_id integer DEFAULT NULL,
+  child_service_id integer DEFAULT NULL,
+  disable_checks enum_boolean DEFAULT NULL,
+  disable_notifications enum_boolean DEFAULT NULL,
+  ignore_soft_states enum_boolean DEFAULT NULL,
+  period_id integer DEFAULT NULL,
+  zone_id integer DEFAULT NULL,
+  assign_filter text DEFAULT NULL,
+  parent_service_by_name character varying(255),
+  PRIMARY KEY (id),
+  CONSTRAINT icinga_dependency_parent_host
+    FOREIGN KEY (parent_host_id)
+    REFERENCES icinga_host (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_parent_service
+    FOREIGN KEY (parent_service_id)
+    REFERENCES icinga_service (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_child_host
+    FOREIGN KEY (child_host_id)
+    REFERENCES icinga_host (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_child_service
+    FOREIGN KEY (child_service_id)
+    REFERENCES icinga_service (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_period
+    FOREIGN KEY (period_id)
+    REFERENCES icinga_timeperiod (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_zone
+    FOREIGN KEY (zone_id)
+    REFERENCES icinga_zone (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+);
+
+CREATE INDEX dependency_parent_host ON icinga_dependency (parent_host_id);
+CREATE INDEX dependency_parent_service ON icinga_dependency (parent_service_id);
+CREATE INDEX dependency_child_host ON icinga_dependency (child_host_id);
+CREATE INDEX dependency_child_service ON icinga_dependency (child_service_id);
+CREATE INDEX dependency_period ON icinga_dependency (period_id);
+CREATE INDEX dependency_zone ON icinga_dependency (zone_id);
+
+
+CREATE TABLE icinga_dependency_inheritance (
+  dependency_id integer NOT NULL,
+  parent_dependency_id integer NOT NULL,
+  weight integer DEFAULT NULL,
+  PRIMARY KEY (dependency_id, parent_dependency_id),
+  CONSTRAINT icinga_dependency_inheritance_dependency
+    FOREIGN KEY (dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_dependency_inheritance_parent_dependency
+    FOREIGN KEY (parent_dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE RESTRICT
+    ON UPDATE CASCADE
+);
+
+CREATE UNIQUE INDEX dependency_inheritance_unique_order ON icinga_dependency_inheritance (dependency_id, weight);
+CREATE INDEX dependency_inheritance_dependency ON icinga_dependency_inheritance (dependency_id);
+CREATE INDEX dependency_inheritance_dependency_parent ON icinga_dependency_inheritance (parent_dependency_id);
+
+
+CREATE TABLE icinga_dependency_states_set (
+  dependency_id integer NOT NULL,
+  property enum_state_name NOT NULL,
+  merge_behaviour enum_set_merge_behaviour NOT NULL DEFAULT 'override',
+  PRIMARY KEY (dependency_id, property, merge_behaviour),
+  CONSTRAINT icinga_dependency_states_set_dependency
+    FOREIGN KEY (dependency_id)
+    REFERENCES icinga_dependency (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE INDEX dependency_states_set_dependency ON icinga_dependency_states_set (dependency_id);
+COMMENT ON COLUMN icinga_dependency_states_set.merge_behaviour IS 'override: = [], extend: += [], blacklist: -= []';
+
+
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (128, NOW());
+  VALUES (144, NOW());
